/*************************************************************************
 ** Functions of visuservServoingTask Codel 
 ** Author: N ouadah
 ** Date: 03 02 2008
 ************************************************************************/

/* standard library*/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/time.h>
#include <time.h>

//extern "C" {
#include "meschach/matrix.h"
#include "meschach/matrix2.h"
//}
// Camera intrinsic parameters (from calibration)
static double focal=1.0,alpha_u=441.36,alpha_v=331.75;	// valeur nominale caméra rackham du haut focal=0.0045;
static 	int Uo=159.05,Vo=122.71;	// centre de l`image

// ----------- Functions --------------:
static MAT*  GetVisualDataInteractionMatrixPix(ICU_VISUAL_DATA visualData, MAT *L);
static MAT*  GetVisualDataInteractionMatrix(ICU_VISUAL_DATA visualData, MAT *L);
static MAT* visuservMatxComb = MNULL;  

/*! previous base time to compute sampling time.*/
static struct timeval previousTime;
static struct timeval endTime;
static VISUSERV_ROBOT_CONFIGURATION_STR previousQ;

/******************************** SIGN ***************************************/
int Sign(double x){
	double i;
	if(x>0) i=1; else if (x<0) i=-1; else i=0;
	return i;
}

/******************************** SAT ***************************************/
double Sat(double x, double phi){
	double y;
	if((x<=phi) & (x>=-phi)) y=x/phi; else y=Sign(x);
	return y;
}
/******************************** SMOOTH ***************************************/
double Smooth(double x, double phi){
	double i;
	if((x<=phi)||(x>=-phi)) i=x/(phi+abs(x)); else i=Sign(x);
	return i;
}
/***************************** Pixelik Interaction-Matrix *****************************/
MAT*  GetVisualDataInteractionMatrixPix(ICU_VISUAL_DATA visualData, MAT *L){
	int i, line,col;

	/* allocate and initialize interaction matrix*/
	if (L==MNULL){	L=m_get(8,3);  }
	
	/* calcul de la matrice interaction*/
	for (i=0, line = 0; i< 4;i++ ){
		col=0;
		L->me[line][col++] = 0.0;
		L->me[line][col++] = (double) (visualData.x[i] / visualData.z[i]);
		L->me[line][col++] = (double) (visualData.x[i] * visualData.y[i]/focal);		
		line++;col=0;
		L->me[line][col++] = (double) (-focal/visualData.z[i]);
		L->me[line][col++] = (double) (visualData.y[i] / visualData.z[i]);
		L->me[line][col++] = (double) (focal+(visualData.y[i] * visualData.y[i]/focal));
		line++;
	}
	
	return L;
}

/*******************  Metric interaction MATRIX L(s,z) *************************/
MAT*  GetVisualDataInteractionMatrix(ICU_VISUAL_DATA visualData, MAT *L){

	int i, line,col;
	double X,Y;

	/* allocate and initialize interaction matrix*/
	if (L==MNULL){	L=m_get(8,3);  }
	
	/* calcul de la matrice interaction*/
	for (i=0, line = 0; i< 4;i++ ){
	// calcul preliminaire des valeurs metrique X et Y	
	X =(double)(focal*(visualData.x[i]-Uo)/alpha_u);
	Y = (double)(focal*(visualData.y[i]-Vo)/alpha_v); 
	//printf("X=%.3f,Y=%.3f |",X,Y);	
	col=0;
	L->me[line][col++]= 0.0;
	L->me[line][col++]=(double) ( X / visualData.z[i]);
	L->me[line][col++]=(double) ( X * Y / focal);		
	line++;col=0;
	L->me[line][col++]=(double) ( -focal / visualData.z[i]);
	L->me[line][col++]=(double) ( Y / visualData.z[i]);
	L->me[line][col++]=(double) ( focal + Y * Y / focal);
	line++;
	}
	//printf("\n");
	return L;
}

/************************************ Set-Combinaison-Matrix **********************************/
STATUS SetCombinaisonMatrix(MAT *C){
	visuservMatxComb=C;
	return OK;
}

/************************************ Get-Combinaison-Matrix **********************************/
STATUS GetCombinaisonMatrix(ICU_VISUAL_DATA visualData, MAT **C){

	MAT *L = MNULL,*M = MNULL;
	int i,j;

	/* allocate and initialize combinaison matrix*/
	if (C==NULL){
		if (visuservMatxComb == MNULL) visuservMatxComb = m_get(3,8);
		C=&visuservMatxComb;
	}else	if (*C== MNULL){
		if (visuservMatxComb == MNULL) visuservMatxComb = m_get(3,8);
		else{
		/*assume that visuServMatxComb still computed*/	*C=visuservMatxComb;
		  return OK;}
		*C=visuservMatxComb;}
        
	/* calcul de la matrice interaction*/
	L= GetVisualDataInteractionMatrix(visualData,MNULL);
	//for (i = 0; i < 8 ; i++){ for (j=0; j < 3; j++) {printf("L[%d,%d]=%lg |",i,j, L->me[i][j]);}
	//printf("\n");}	printf("--L ok --\n");
	
	// combinaison matrix computation C = inv(L'*L) * L'  
	M = m_get(3,3); 
  	M = mtrm_mlt(L,L,MNULL);
	M = m_inverse(M,MNULL);
	m_mlt(M, m_transp(L,MNULL), *C);

	/* test des calcul N = C*L
	m_mlt((*C),L,M);
	printf("I = \n");for (i = 0; i < 3 ; i++){
	for (j=0; j < 3; j++) printf(" %lg |",M->me[i][j]); printf("\n");}*/
        	
   	/*garbage */
	m_free(L);m_free(M);

	return OK;
}

/************************************ Get-Lambda-Matrix **********************************/
MAT* GetLambdaMAT(double lambda[], int dim, MAT *out){
	int i;
	
	/*alloc/create vec out*/
	if ( (out==MNULL) || (out->n!=out->m) || ( out->m < dim ))
		out = m_resize(out,dim,dim);
	out=m_zero(out);
	for (i=0;i<dim;i++ )
		out->me[i][i]=-lambda[i];
	return out;
}

/**************************** Get-Jacobian ******************************/
STATUS GetRobotJacobian( double pan,MAT **J_pl){

 	/* allocate and initialize jacobian Matrix*/
	if (J_pl==NULL){
		printf("getRobotJacobian: bad NULL parameter \n");
		return ERROR;	}

	if(*J_pl==MNULL) *J_pl    = m_get(3,3);
	(*J_pl)->me[0][0]=-sin(pan);
	(*J_pl)->me[0][1]= VISUSERV_Cx+VISUSERV_Dx*cos(pan);
	(*J_pl)->me[0][2]= VISUSERV_Cx;

	(*J_pl)->me[1][0]= cos(pan);
	(*J_pl)->me[1][1]= -VISUSERV_Cy+VISUSERV_Dx*sin(pan);
	(*J_pl)->me[1][2]= -VISUSERV_Cy;
	
	(*J_pl)->me[2][0]=0;	
	(*J_pl)->me[2][1]=-1.0;	
	(*J_pl)->me[2][2]=-1.0;	

	return OK;
}

/**************************** Get-Inverse-Jacobian ******************************/
STATUS GetInverseRobotJacobian( double pan,MAT **J_pl)
{

 	/* allocate and initialize jacobian Matrix*/
	if (J_pl==NULL){
		printf("getRobotJacobian: bad NULL parameter \n");
		return ERROR;	}
	
	if(*J_pl==MNULL) *J_pl = m_get(3,3);
	(*J_pl)->me[0][0]=-sin(pan);
	(*J_pl)->me[0][1]= cos(pan);
	(*J_pl)->me[0][2]=-VISUSERV_Cy*cos(pan)-VISUSERV_Cx*sin(pan);

	(*J_pl)->me[1][0]= cos(pan)/VISUSERV_Dx;
	(*J_pl)->me[1][1]= sin(pan)/VISUSERV_Dx;
	(*J_pl)->me[1][2]= (-VISUSERV_Cy*sin(pan)+VISUSERV_Cx*cos(pan))/VISUSERV_Dx;
	
	(*J_pl)->me[2][0]=-cos(pan)/VISUSERV_Dx;	
	(*J_pl)->me[2][1]=-sin(pan)/VISUSERV_Dx;	
	(*J_pl)->me[2][2]=-(-VISUSERV_Cy*sin(pan)+VISUSERV_Cx*cos(pan)+VISUSERV_Dx)/VISUSERV_Dx;	
	
	return OK;
}

/*************************************** Get-TaskFunction *************************************/
STATUS GetVisuservTaskFunc(ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s, MAT *C, VEC ** taskFunc){
	
	VEC * error;  
	int i=0, j,k;
	double X,Xr,Y,Yr;

	// allocation 
	if (*taskFunc == NULL) *taskFunc = v_get(3);
	error = v_get(8);
	
	//Substraction error = s-s_ref
	for (i=0, j=0; i<4; i++, j+=2 ){
	X=(double)(focal*(s.x[i]-Uo)/alpha_u);   Xr=(double)(focal*(s_ref.x[i]-Uo)/alpha_u);   
	Y=(double)(focal*(s.y[i]-Vo)/alpha_v);   Yr=(double)(focal*(s_ref.y[i]-Vo)/alpha_v);    
	error->ve[j]=(double)(X-Xr); 		  error->ve[j+1]=(double)(Y-Yr);
	//printf("\n \n %d-->  u= %.1f , X= %lg , Xr= %lg || v= %.1f , Y= %lg , Yr=  %lg \n",i+1,s.x[i],X,Xr,s.y[i],Y,Yr);  
	}
	//printf("%.4f,%.4f->",X-Xr,Y-Yr);

	//taskFunc = C * error
	mv_mlt(C,error,*taskFunc);

	v_free(error);
	return OK;
}
/*************************************** Get-zEstim *************************************/
VEC* GetZEstim(double pan, ICU_VISUAL_DATA s, ICU_VISUAL_DATA s_old, VISUSERV_ROBOT_CONTROLS_STR dot_q_old,VEC *z){
	
//	float z[4]; /*profondeur estime*/
	float Te = 0.05; /*periode d echantillonage en s*/
	int i;

	// init variables 
	z=v_get(4);
	for (i = 0; i<4; i++){
	s.x[i] 	=(double)(focal*(s.x[i]-Uo)/alpha_u);
	s.y[i] 	= (double)(focal*(s.y[i]-Vo)/alpha_v); 
	s_old.x[i] =(double)(focal*(s_old.x[i]-Uo)/alpha_u);
	s_old.y[i] = (double)(focal*(s_old.y[i]-Vo)/alpha_v); 
	}
	printf("Zes= %.4f, %.4f,%.4f, %.4f, %.2f ",s.x[0],s.y[0],s_old.x[0],s_old.y[0],s.z[0]);

	for (i = 0; i<4; i++){
	
		/*First case*/
		if ((dot_q_old.wr != 0) && (dot_q_old.p != 0) && (dot_q_old.wr != dot_q_old.p)){
			double alpha1, alpha2, alpha3, alpha4, alpha5;
		
		alpha1 = VISUSERV_Dx*sin(pan) + (dot_q_old.vr/dot_q_old.wr)*cos(pan) - VISUSERV_Cy;
		alpha2 = VISUSERV_Dx*cos(pan) - (dot_q_old.vr/dot_q_old.wr)*sin(pan) + VISUSERV_Cx;
		alpha3 = -VISUSERV_Dx*cos(pan + dot_q_old.p*Te) - (dot_q_old.vr/dot_q_old.wr)*sin(pan + dot_q_old.p*Te) - VISUSERV_Cx;
		alpha4 = s.y[i]/focal*sin((dot_q_old.wr+dot_q_old.p)*Te) + cos((dot_q_old.wr+dot_q_old.p)*Te);
		alpha5 = alpha1*sin((dot_q_old.wr+dot_q_old.p)*Te) + alpha2*cos((dot_q_old.wr+dot_q_old.p)*Te) + alpha3;
		z->ve[i] = (-alpha5*s_old.x[i]) / (alpha4*s.x[i] - s_old.x[i]);
			}

		//Second case
		if ((dot_q_old.wr == 0) && (dot_q_old.p != 0)){
			double alpha1, alpha2, alpha3, alpha4, alpha5;
		
		alpha1 = -(dot_q_old.vr/(2*dot_q_old.p))*cos(pan) - VISUSERV_Cy;
		alpha2 = - (dot_q_old.vr/(2*dot_q_old.p))*sin(pan) + VISUSERV_Cx;
		alpha3 = -dot_q_old.vr*Te*cos(pan + dot_q_old.p*Te) + (dot_q_old.vr/(2*dot_q_old.p))*sin(pan + dot_q_old.p*Te) - VISUSERV_Cx;
		alpha4 = s.y[i]/focal*sin(dot_q_old.p*Te) + cos(dot_q_old.p*Te);
		alpha5 = alpha1*sin(dot_q_old.p*Te) + alpha2*cos(dot_q_old.p*Te) + alpha3;
		z->ve[i] = (-alpha5*s_old.x[i]) / (alpha4*s.x[i] - s_old.x[i]);
		}
	
		//Third case
		if ((dot_q_old.wr == dot_q_old.p) && (dot_q_old.wr != 0)){
			double alpha;
	
	alpha = -(dot_q_old.vr/dot_q_old.p)*(sin(pan + dot_q_old.p*Te) - sin(pan)) + (dot_q_old.wr/dot_q_old.p)*VISUSERV_Dx*(cos(pan + dot_q_old.p*Te) - cos(pan));
		z->ve[i] = (-alpha*s_old.x[i]) / (s.x[i] - s_old.x[i]);
		}
	
		//Forth case
		if ((dot_q_old.wr == dot_q_old.p) && (dot_q_old.wr == 0)){
			double alpha;
	
			alpha = -dot_q_old.vr*cos(pan)*Te;
	
			z->ve[i] = (-alpha*s_old.x[i]) / (s.x[i] - s_old.x[i]);
		}
	
	
	}
	return z;
}

/*************************************** BASIC VS : Classic Controller (ds/dt = 0)***********************************/
STATUS vsController(double pan,double pan_old, MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams,VISUSERV_ROBOT_CONTROLS_STR dot_q_old, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint=VNULL,*taskFunc=VNULL,*Z=VNULL;  
	MAT *Ji=MNULL, *mLambda=MNULL, *JM=MNULL;    
	int i,k,j;
	static int  d1=0,d2=0;
	double Lbda[3];

	// z estimation: 	Z=GetZEstim(pan_old, s, s_old, dot_q_old, VNULL);
	
	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Bst|");
	else if (visuservParams.vs_mode==VISUSERV_BOTH) printf("Bbo=%.2f|"); 
	else printf("Bva=%.2f|"); 	
	
	//Get the taskFunction 
	if (GetVisuservTaskFunc(s_ref,s,C,&taskFunc) != OK) return ERROR;
	printf("CTF:%.2f-> ",v_norm2(taskFunc));
	for (k=0; k<3; k++)  {state->TF[k]= taskFunc->ve[k]; printf("%.2f |",taskFunc->ve[k]);}
	        	
	/* if we are close to the goal or not*/
	if (v_norm2(taskFunc)<visuservParams.epsi) { 
	  dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; 	state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(taskFunc));d1=1;}
          	v_free(taskFunc);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	/*compute the jacobian*/
	GetInverseRobotJacobian(pan,&Ji ); 
	//for (i = 0; i < 3 ; i++){for (j=0; j < 3; j++) printf("| %.2f ",Ji->me[i][j]);}	printf("\n");
		
	// Lambda tuning and Lambda matrix computation :
	Lbda[0]=visuservParams.lambda[0];     Lbda[1]=visuservParams.lambda[1];    Lbda[2]=visuservParams.lambda[2];	
	//if(fabs(pan)>0.02){Lbda[0]=(double)(fabs(sin(pan))*visuservParams.lambda[0]);printf("@P");} 
	
	mLambda=GetLambdaMAT(Lbda, 3, MNULL);
	//for (i = 0; i < 3 ; i++){ printf(" %lg|", mLambda->me[i][i]);} 
	
	//initialize and compute dot_q : -J^-1 * LAMBDA * C * (s -s_ref)
	qpoint = v_get(3);	qpoint = mv_mlt(m_mlt(Ji,mLambda,JM),taskFunc, VNULL );	
        
	// linear velocity control (with respec to target scale)
  	if (fabs(s.scale- s_ref.scale)>.15) qpoint->ve[0] = -0.25*(s.scale- s_ref.scale);  else  qpoint->ve[0] = -0;	  

	printf("Sc= %.3f, V= %lg, W= %.3f, P= %.3f|",s.scale- s_ref.scale, qpoint->ve[0],qpoint->ve[1],qpoint->ve[2]);

	// Software limits
	if (qpoint->ve[0]> 0.05)   {qpoint->ve[0] = 0.05;  printf ("v+");} 
	if (qpoint->ve[0]<-0.05)   {qpoint->ve[0] = -0.05; printf ("v-");} 
	if (qpoint->ve[1]> 0.25)  {qpoint->ve[1] = 0.25;  printf ("w+");} 
	if (qpoint->ve[1]< -0.25) {qpoint->ve[1] = -0.25; printf ("w-");} 
	if (qpoint->ve[2]> 0.25)  {qpoint->ve[2] = 0.25;  printf ("p+");} 
	if (qpoint->ve[2]< -0.25) {qpoint->ve[2] = -0.25; printf ("p-");} 
		 
	dot_q->vr  = qpoint->ve[0]; 	dot_q->wr = qpoint->ve[1];    dot_q->p  = qpoint->ve[2];

	/*garbage*/
	m_free(Ji);	m_free(JM); 	m_free(mLambda);
	v_free(qpoint);	v_free(taskFunc);
	
	return OK;
}

/****************************************** Sliding Modes Controller (old) *********************************/
/****													  ****/
STATUS smController(double pan,MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old,VISUSERV_ROBOT_CONTROLS_STR qp_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint,*qpoint1,*qpoint2,*TF=VNULL,*TFP=VNULL,*TFP1=VNULL,*TFP2=VNULL,*STF=VNULL,*ds=VNULL,*ds2=VNULL,*ds1=VNULL; 
	VEC *qp=VNULL, *ks=VNULL,*Eta=VNULL,*SSP=VNULL,*Eps=VNULL,*Cost=VNULL,*Simp=VNULL,*JSmax=VNULL,*JJSm=VNULL; // ds=s_old-s 
	MAT *Ji=MNULL,*J=MNULL,*JS=MNULL,*L=MNULL, *K=MNULL;    // K : gains matrix for sliding modes controller
	int i,k,k1b=-9,k1g=-9,Ksi1=1.1,Ksi2=0.9,j,js=0,cor=9;
	static int  d1=0,d2=0,kk=0;
	double dt=0.04,beta=0.0,gamma=0.0,beta1b=0.0,gamma1b=0.0,beta1g=0.0,gamma1g=0.0,alpha=0.0,Phi=2.0,q_dot_max=0.4;
	VISUSERV_VISUAL_SERVOING_PARAMS_STR VSCParams;

	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Sct|");
	else if (visuservParams.vs_mode==VISUSERV_BOTH) printf("Sbo|"); 
	else printf("Sva|"); 	
	
	//Get the taskFunction (TF) ans it's sign (STF)
	if (GetVisuservTaskFunc(s_ref,s,C,&TF) != OK) return ERROR;
	printf("STF:%.2f:",v_norm2(TF)); STF = v_get(3);
	for (k=0; k<3; k++)  {state->TF[k]= TF->ve[k]; STF->ve[k]=Sat(TF->ve[k],Phi);printf("%.2f/%.2f|",TF->ve[k],STF->ve[k]); }
	        	
	// if we are close to the goal or not
	if (v_norm2(TF)<visuservParams.epsi) { 
	  dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; 	state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(TF));d1=1;}
          	v_free(TF);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	// Compute -ds/dt = s_old - s /dt (en metres): not used for the moment
        ds = v_get(8); for (i=0, j=0; i<4; i++, j+=2 ){
	ds->ve[j]      = (double)(0*focal*(s_old.x[i]-s.x[i])/(alpha_u*dt));    
	ds->ve[j+1]    = (double)(0*focal*(s_old.y[i]-s.y[i])/(alpha_v*dt));
	//printf("DST: %.2f , %.2f | ",ds->ve[j],ds->ve[j+1]);  
	}

	//compute the jacobian
	GetInverseRobotJacobian(pan,&Ji ); 	GetRobotJacobian(pan,&J );
	//for (i = 0; i < 3 ; i++){for (j=0; j < 3; j++) printf("| %.2f ",Ji->me[i][j]);}	printf("\n");
		
	// JS and Eta computation so that Eta = JS*ks :
	JS = m_get(3,3);Eta = v_get(3);ks = v_get(3);Eps = v_get(3);Cost = v_get(3);Simp = v_get(4);JSmax = v_get(3);JJSm = v_get(3);
	for (i=0; i<3; i++)  {  JSmax->ve[i]=-9999; ks->ve[i]=visuservParams.lambda[i]; 
				for (j=0; j<3; j++)  {JS->me[i][j]= J->me[i][j]*STF->ve[i]*STF->ve[j]; 
					             if (JSmax->ve[i] <JS->me[i][j]) {JJSm->ve[i]==j; JSmax->ve[i]=JS->me[i][j];}}
			     }

      	// Sliding mode condition -> S21 sign 
	if (STF->ve[1]*STF->ve[2]>0) { printf("Cor39-> "); cor=9;  goto l2;  } 

        // k1 and k2 computation under sliding condition ss_dot<0 (k0=cte)
	beta=-(JS->me[0][0]*ks->ve[0]+JS->me[0][1]*ks->ve[1])/(JS->me[0][2]);
	gamma=-(JS->me[1][0]*ks->ve[0]+JS->me[1][1]*ks->ve[1])/(JS->me[1][2]);
	
	// case I: S20 ++ ==>  max(beta,gamma) < k2 < k1
	if (STF->ve[2]*STF->ve[0]>0 & JS->me[1][2]!=0) {
	if (ks->ve[1]>=max(gamma,beta) & max(gamma,beta)>0) {printf("Cor11-> ");cor=-9; ks->ve[2]=0.5*(ks->ve[1]+max(gamma,beta)); }
	else if (ks->ve[1]>max(gamma,beta) & max(gamma,beta)<=0) {printf("Cor12-> ");cor=-9; ks->ve[2]=0.95*ks->ve[1]; }
	else if (ks->ve[1]<max(gamma,beta) & max(gamma,beta)<= q_dot_max) {printf("Cor13-> ");cor=-9; ks->ve[1]=1.2*max(gamma,beta); ks->ve[2]=0.9*ks->ve[1];}
	else {  printf("Cor19-> "); cor=9;  goto l2;}
				     }
	// case II: S20 -- ==> min(beta,k1) < k2 < gamma 
	if (STF->ve[2]*STF->ve[0]<0 & JS->me[1][2]!=0) {
	if (gamma>=0 & gamma<ks->ve[1] & beta>0) { printf("Cor21-> "); cor=-9; ks->ve[2]=0.5*(min(ks->ve[1],beta)+ gamma); }
	else if (beta>0 & gamma<0) {	printf("Cor22-> "); cor=-9;  ks->ve[2]=0.95*ks->ve[1]; }
	else if (beta>0 & gamma>0 & gamma>min(ks->ve[1],beta)) { printf("Cor23-> "); cor=-9;   ks->ve[1]=1.1*gamma; ks->ve[2]=0.95*ks->ve[1]; }
	else { printf("Cor29-> "); cor=9;  goto l2;}
				     }
	// Eta computation
l2:	Eta = mv_mlt(JS,ks, VNULL);

	// sliding mode controller q_dot = - [ J^-1 * C * ds/dt + K * Sign(TaskFunction)] 
        for (k=0; k<3; k++) {VSCParams.lambda[k] = ks->ve[k];}
	if (cor>0) K=GetLambdaMAT(visuservParams.lambda, 3, MNULL); else K=GetLambdaMAT(VSCParams.lambda, 3, MNULL);
	qpoint = v_get(3); qpoint1 = v_get(3); qpoint2 = v_get(3);
	//qpoint1 = mv_mlt(m_mlt(Ji,C,MNULL),ds, VNULL);	v_add(qpoint1 ,qpoint2 ,qpoint);
	qpoint = mv_mlt(K,STF, VNULL);	
	
	// compute lyapunov function derivate Vpoint = (Sigma)T*(Sigmapoint) :
	TFP = v_get(3); TFP1 = v_get(3);TFP2 = v_get(3);SSP = v_get(3);
	mv_mlt(J,qpoint,TFP); //mv_mlt(C,ds,TFP2); v_add(TFP1,TFP2,TFP);	
	for (k=0; k<3; k++)  {state->TFP[k]= TFP->ve[k];SSP->ve[k]=(TF->ve[k])*(TFP->ve[k]); state->KSM[k]=Eta->ve[k];}

	printf(" Eta: %.1f/ %.1f/ %.1f | k:%.3f/%.3f/%.3f |V= %.3f|W= %.3f|P= %.3f|",Eta->ve[0],Eta->ve[1],Eta->ve[2],ks->ve[0],ks->ve[1],ks->ve[2],qpoint->ve[0],qpoint->ve[1],qpoint->ve[2]);
	
	// Software limits
	if (qpoint->ve[0]>0.08) {qpoint->ve[0] = 0.08;} 
	if (qpoint->ve[0]<=-0.08) {qpoint->ve[0] = -0.08;} 
	if (qpoint->ve[1]>0.25) {qpoint->ve[1] = 0.25;} 
	if (qpoint->ve[1]<-0.25) {qpoint->ve[1] = -0.25;} 
	if (qpoint->ve[2]>0.25) {qpoint->ve[2] = 0.25; } 
	if (qpoint->ve[2]<-0.25) {qpoint->ve[2] = -0.25;} 
			 
	dot_q->vr=qpoint->ve[0]; 	dot_q->wr=qpoint->ve[1];    dot_q->p=qpoint->ve[2];   dot_q->t=ks->ve[1];

	//garbage
	m_free(K);m_free(L);m_free(Ji); m_free(J); m_free(JS);
	v_free(TF); v_free(STF); v_free(TFP); v_free(TFP1);v_free(TFP2);v_free(ds); v_free(SSP);v_free(Simp);
	v_free(qpoint); v_free(qpoint1); v_free(qpoint2);v_free(Eta);v_free(ks);v_free(Eps);v_free(Cost);v_free(JSmax);v_free(JJSm);
	
	return OK;
}
/****													  ****/
/************************************* New Sliding Modes Controller (K=Beta*(J)-1) ***************************/
/****													  ****/
STATUS sm3Controller(double pan,MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint,*qpoint1,*qpoint2,*TF=VNULL,*TFP=VNULL,*TFP1=VNULL,*TFP2=VNULL,*STF=VNULL,*ds=VNULL,*ds2=VNULL,*ds1=VNULL; 
	VEC *qp=VNULL, *ks=VNULL,*Eta=VNULL,*SSP=VNULL,*Eps=VNULL,*Cost=VNULL,*Simp=VNULL,*JSmax=VNULL,*JJSm=VNULL; // ds=s_old-s 
	MAT *Ji=MNULL,*J=MNULL,*JS=MNULL,*L=MNULL, *K=MNULL, *Bet=MNULL;    // K : gains matrix for sliding modes controller
	int i,k,k1b=-9,k1g=-9,Ksi1=1.1,Ksi2=0.9,j,js=0,cor=9;
	static int  d1=0,d2=0,kk=0;
	double dt=0.04,beta=0.0,gamma=0.0,beta1b=0.0,gamma1b=0.0,beta1g=0.0,gamma1g=0.0,alpha=0.0,Phi=2.0,q_dot_max=0.4;
	VISUSERV_VISUAL_SERVOING_PARAMS_STR VSCParams;

	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Ct|");
	else  {printf("This control mode requires Constant Interaction Matrix !\n"); return END; } 	
		
	//Get the taskFunction (TF) ans it's sign (STF)
	if (GetVisuservTaskFunc(s_ref,s,C,&TF) != OK) return ERROR;
	printf("TF:%.2f:",v_norm2(TF)); STF = v_get(3);
	for (k=0; k<3; k++) {	state->TF[k]= TF->ve[k]; 
			if (visuservParams.sm_type==VISUSERV_SLIDING_1_SIGN) 
				{ printf("Sg|"); STF->ve[k]=Sign(TF->ve[k]);}
				else if (visuservParams.sm_type==VISUSERV_SLIDING_2_SAT) 
					{ printf("Sa|"); STF->ve[k]=Sat(TF->ve[k],Phi);}
					else  	{ printf("Sm|"); STF->ve[k]=Smooth(TF->ve[k],Phi);}
			printf("%.2f/%.2f|",TF->ve[k],STF->ve[k]);}
	        	
	// if we are close to the goal or not
	if (v_norm2(TF)<visuservParams.epsi) { 
	  dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; 	state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(TF));d1=1;}
          	v_free(TF);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	//compute the jacobian
	GetInverseRobotJacobian(pan,&Ji ); 	GetRobotJacobian(pan,&J );
	//for (i = 0; i < 3 ; i++){for (j=0; j < 3; j++) printf("| %.2f ",Ji->me[i][j]);}	printf("\n");
		
	// JS and Eta computation so that Eta = JS*ks :
	JS = m_get(3,3);Eta = v_get(3);ks = v_get(3);Eps = v_get(3);Cost = v_get(3);Simp = v_get(4);JSmax = v_get(3);JJSm = v_get(3);
	for (i=0; i<3; i++)  {  JSmax->ve[i]=-9999; ks->ve[i]=visuservParams.lambda[i]; 
				for (j=0; j<3; j++)  {JS->me[i][j]= J->me[i][j]*STF->ve[i]*STF->ve[j]; 
					             if (JSmax->ve[i] <JS->me[i][j]) {JJSm->ve[i]==j; JSmax->ve[i]=JS->me[i][j];}}
			     }

      	// Sliding mode condition -> S21 sign 
	if (STF->ve[1]*STF->ve[2]>0) { printf("Cor39->"); cor=9;  goto l2;  } 

        // k1 and k2 computation under sliding condition ss_dot<0 (k0=cte)
	beta=-(JS->me[0][0]*ks->ve[0]+JS->me[0][1]*ks->ve[1])/(JS->me[0][2]);
	gamma=-(JS->me[1][0]*ks->ve[0]+JS->me[1][1]*ks->ve[1])/(JS->me[1][2]);
	
	// case I: S20 ++ ==>  max(beta,gamma) < k2 < k1
	if (STF->ve[2]*STF->ve[0]>0 & JS->me[1][2]!=0) {
	if (ks->ve[1]>=max(gamma,beta) & max(gamma,beta)>0) {printf("Cor11-> ");cor=-9; ks->ve[2]=0.5*(ks->ve[1]+max(gamma,beta)); }
	else if (ks->ve[1]>max(gamma,beta) & max(gamma,beta)<=0) {printf("Cor12-> ");cor=-9; ks->ve[2]=0.95*ks->ve[1]; }
	else if (ks->ve[1]<max(gamma,beta) & max(gamma,beta)<= q_dot_max) {printf("Cor13-> ");cor=-9; ks->ve[1]=1.2*max(gamma,beta); ks->ve[2]=0.9*ks->ve[1];}
	else {  printf("Cor19-> "); cor=9;  goto l2;}
				     }
	// case II: S20 -- ==> min(beta,k1) < k2 < gamma 
	if (STF->ve[2]*STF->ve[0]<0 & JS->me[1][2]!=0) {
	if (gamma>=0 & gamma<ks->ve[1] & beta>0) { printf("Cor21-> "); cor=-9; ks->ve[2]=0.5*(min(ks->ve[1],beta)+ gamma); }
	else if (beta>0 & gamma<0) {	printf("Cor22-> "); cor=-9;  ks->ve[2]=0.95*ks->ve[1]; }
	else if (beta>0 & gamma>0 & gamma>min(ks->ve[1],beta)) { printf("Cor23-> "); cor=-9;   ks->ve[1]=1.1*gamma; 
									ks->ve[2]=0.95*ks->ve[1]; }
	else { printf("Cor29-> "); cor=9;  goto l2;}
			     }
	// Eta computation
l2:	Eta = mv_mlt(JS,ks, VNULL);

	// sliding mode controller q_dot = - [ J^-1 * C * ds/dt + K * Sign(TaskFunction)] 
        for (k=0; k<3; k++) {VSCParams.lambda[k] = ks->ve[k];}
	K=GetLambdaMAT(visuservParams.lambda, 3, MNULL); 
	qpoint = v_get(3); qpoint1 = v_get(3); qpoint2 = v_get(3);
	//qpoint = mv_mlt(K,STF, VNULL);	
	qpoint = mv_mlt(m_mlt(Ji,K,Bet),STF, VNULL );		

	// compute lyapunov function derivate Vpoint = (Sigma)T*(Sigmapoint) :
	TFP = v_get(3); TFP1 = v_get(3);TFP2 = v_get(3);SSP = v_get(3);
	mv_mlt(J,qpoint,TFP); //mv_mlt(C,ds,TFP2); v_add(TFP1,TFP2,TFP);	
	for (k=0; k<3; k++)  {state->TFP[k]= TFP->ve[k];SSP->ve[k]=(TF->ve[k])*(TFP->ve[k]); state->KSM[k]=K->me[k][k];}

	printf("pan=%.1f|SSP: %.3f |V= %.3f|W= %.3f|P= %.3f",180*pan/3.1415,(SSP->ve[0]+SSP->ve[1]+SSP->ve[2]),qpoint->ve[0],qpoint->ve[1],qpoint->ve[2]);
	
	// Software limits
	if (qpoint->ve[0]>0.15) {qpoint->ve[0] = 0.15;} 
	if (qpoint->ve[0]<=-0.15) {qpoint->ve[0] = -0.15;} 
	if (qpoint->ve[1]>0.2) {qpoint->ve[1] = 0.2;} 
	if (qpoint->ve[1]<-0.2) {qpoint->ve[1] = -0.2;} 
	if (qpoint->ve[2]>0.2) {qpoint->ve[2] = 0.2; } 
	if (qpoint->ve[2]<-0.2) {qpoint->ve[2] = -0.2;} 
			 
	dot_q->vr=qpoint->ve[0]; 	dot_q->wr=qpoint->ve[1];    dot_q->p=qpoint->ve[2];   dot_q->t=ks->ve[1];

	//garbage
	m_free(K);m_free(L);m_free(Ji); m_free(J); m_free(JS);m_free(Bet);
	v_free(TF); v_free(STF); v_free(TFP); v_free(TFP1);v_free(TFP2);v_free(ds); v_free(SSP);v_free(Simp);
	v_free(qpoint); v_free(qpoint1); v_free(qpoint2);v_free(Eta);v_free(ks);v_free(Eps);v_free(Cost);v_free(JSmax);v_free(JJSm);
	
	return OK;
}

/*************************************** New Sliding Modes Controller (ADV paper) *****************************/
/****													  ****/
STATUS sm4Controller(double pan,MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint,*TF=VNULL,*TFP=VNULL,*TFP1=VNULL,*TFP2=VNULL,*STF=VNULL,*ds=VNULL; 
	VEC *qp=VNULL, *ks=VNULL,*Eta=VNULL,*SSP=VNULL,*Eps=VNULL,*Cost=VNULL,*Simp=VNULL,*JJSm=VNULL; // ds=s_old-s 
	MAT *Ji=MNULL,*J=MNULL,*JS=MNULL,*L=MNULL, *K=MNULL, *Beta=MNULL;    // K : gains matrix for sliding modes controller
	int i,k,k1b=-9,k1g=-9,Ksi1=1.1,Ksi2=0.9,j,js=0,cor=9;
	static int  d1=0,d2=0,kk=0;
	double dt=0.04,beta=0.0,gamma=0.0,beta1b=0.0,gamma1b=0.0,beta1g=0.0,gamma1g=0.0,alpha=0.0,Phi=2.0,q_dot_max=0.4;
	VISUSERV_VISUAL_SERVOING_PARAMS_STR VSCParams;

	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Ct|");
	else  {printf("This control mode requires Constant interaction matrix !"); return ERROR; } 	
	
	//Get the taskFunction (TF) ans it's sign (STF)
	if (GetVisuservTaskFunc(s_ref,s,C,&TF) != OK) return ERROR;
	printf("STF:%.2f:",v_norm2(TF)); STF = v_get(3);
	for (k=0; k<3; k++) {	state->TF[k]= TF->ve[k]; 
			if (visuservParams.sm_type==VISUSERV_SLIDING_1_SIGN) 
				{ printf("Sgn|"); STF->ve[k]=Sign(TF->ve[k]);}
				else if (visuservParams.sm_type==VISUSERV_SLIDING_2_SAT) 
					{ printf("Sat|"); STF->ve[k]=Sat(TF->ve[k],Phi);}
					else  	{ printf("Smt|"); STF->ve[k]=Smooth(TF->ve[k],Phi);}
			printf("%.2f/%.2f|",TF->ve[k],STF->ve[k]);}
	        	
	/* if we are close to the goal or not*/
	if (v_norm2(TF)<visuservParams.epsi)  {dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(TF));d1=1;}
          	v_free(TF);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	/*compute the jacobian*/
	GetInverseRobotJacobian(pan,&Ji ); 	GetRobotJacobian(pan,&J );
		
	// New sliding mode controller q_dot = - Beta [ C * L * J] -1 * Sign(TaskFunction) 
        //if (cor>0) K=GetLambdaMAT(visuservParams.lambda, 3, MNULL); else K=GetLambdaMAT(VSCParams.lambda, 3, MNULL);
	qpoint = v_get(3); JS = m_get(3,3);
	Beta=GetLambdaMAT(visuservParams.lambda, 3, MNULL);
	//JS=m_mlt(Beta,Ji,MNULL);
	qpoint = mv_mlt(Beta,STF, VNULL);	
	//for (k=0; k<3; k++) {qpoint->ve[k] = visuservParams.lambda[k]/10.0;}

	// compute lyapunov function derivate sigma*sigma_dot :
	TFP = v_get(3); TFP1 = v_get(3);TFP2 = v_get(3);SSP = v_get(3);
	mv_mlt(J,qpoint,TFP); //mv_mlt(C,ds,TFP2); v_add(TFP1,TFP2,TFP);	
	for (k=0; k<3; k++)  {state->TFP[k]= TFP->ve[k];SSP->ve[k]=(TF->ve[k])*(TFP->ve[k]); state->KSM[k]=ks->ve[k];}

	printf(" SSP: %.1f/ %.1f/ %.1f ||V= %.3f|W= %.3f|P= %.3f|",SSP->ve[0],SSP->ve[1],SSP->ve[2],qpoint->ve[0],qpoint->ve[1],qpoint->ve[2]);
	
	// Software limits
	if (qpoint->ve[0]>0.08) {qpoint->ve[0] = 0.08;/*printf ("v+");*/} 
	if (qpoint->ve[0]<=-0.08) {qpoint->ve[0] = -0.08;/*printf ("v-");*/} 
	if (qpoint->ve[1]>0.25) {qpoint->ve[1] = 0.25;/*printf ("w+");*/} 
	if (qpoint->ve[1]<-0.25) {qpoint->ve[1] = -0.25;/*printf ("w-");*/} 
	if (qpoint->ve[2]>0.25) {qpoint->ve[2] = 0.25; /*printf ("p+");*/} 
	if (qpoint->ve[2]<-0.25) {qpoint->ve[2] = -0.25;/*printf ("p-");*/} 
			 
	dot_q->vr=qpoint->ve[0]; 	dot_q->wr=qpoint->ve[1];    dot_q->p=qpoint->ve[2];   dot_q->t=ks->ve[1];

	/*garbage*/
	m_free(K);m_free(L);m_free(Ji); m_free(J); m_free(JS);m_free(Beta);
	v_free(TF); v_free(STF); v_free(TFP); v_free(TFP1);v_free(TFP2);v_free(ds); v_free(SSP);v_free(Simp);
	v_free(qpoint); v_free(Eta);v_free(ks);v_free(Eps);v_free(Cost);v_free(JJSm);
	
	return OK;
}

/****													****/
/**************************** COMBINED : Sliding modes combined with classical VS ******************************/
/****													****/
STATUS sm2Controller(double pan,MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old,VISUSERV_ROBOT_CONTROLS_STR qp_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint,*qpoint1,*qpoint2,*TF=VNULL,*TFP=VNULL,*TFP1=VNULL,*TFP2=VNULL,*STF=VNULL,*SATF=VNULL,*ds=VNULL,*ds2=VNULL,*ds1=VNULL; 
	VEC *qp=VNULL, *ks=VNULL,*Eta=VNULL,*SSP=VNULL,*Eps=VNULL,*Cost=VNULL,*Simp=VNULL,*JSmax=VNULL,*JJSm=VNULL; // ds=s_old-s STF=Sign(TF)
	MAT *Ji=MNULL,*J=MNULL,*JS=MNULL,*JSA=MNULL,*L=MNULL, *K=MNULL;    // K : gains matrix for sliding modes controller
	int i,k,k1b=-9,k1g=-9,Ksi1=1.1,Ksi2=0.9,j,js=0,cor=9;
	static int  d1=0,d2=0,kk=0;
	double dt=0.04,beta=0.0,gamma=0.0,beta1b=0.0,gamma1b=0.0,beta1g=0.0,gamma1g=0.0,alpha=0.0,Phi=2.0,q_dot_max=0.4;
	VISUSERV_VISUAL_SERVOING_PARAMS_STR VSCParams;

	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Cst|");
	else if (visuservParams.vs_mode==VISUSERV_BOTH) printf("Cbo|"); 
	else printf("Cva|"); 	
	
	//Get the taskFunction (TF) ans it's sign (STF)
	if (GetVisuservTaskFunc(s_ref,s,C,&TF) != OK) return ERROR;
	printf("BTF:%.2f:",v_norm2(TF)); STF = v_get(3); SATF = v_get(3);
	for (k=0; k<3; k++)  {state->TF[k]= TF->ve[k]; STF->ve[k]=Sign(TF->ve[k]); SATF->ve[k]=Sat(TF->ve[k],Phi); printf("%.1f|",STF->ve[k]); }
	        	
	/* if we are close to the goal or not*/
	if (v_norm2(TF)<visuservParams.epsi) { 
	  dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; 	state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(TF));d1=1;}
          	v_free(TF);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	/* Compute -ds/dt = s_old - s /dt (en metres): not used for the moment*/
        ds = v_get(8); for (i=0, j=0; i<4; i++, j+=2 ){
	ds->ve[j]      = (double)(0*focal*(s_old.x[i]-s.x[i])/(alpha_u*dt));    
	ds->ve[j+1]    = (double)(0*focal*(s_old.y[i]-s.y[i])/(alpha_v*dt));
	//printf("DST: %.2f , %.2f | ",ds->ve[j],ds->ve[j+1]);  
	}

	/*compute the jacobian*/
	GetInverseRobotJacobian(pan,&Ji ); 	GetRobotJacobian(pan,&J );
	//for (i = 0; i < 3 ; i++){for (j=0; j < 3; j++) printf("| %.2f ",Ji->me[i][j]);}	printf("\n");
		
	// JS and Eta computation so that Eta = JS*ks :
	JS = m_get(3,3);JSA = m_get(3,3); Eta = v_get(3); ks = v_get(3); Eps = v_get(3); Simp = v_get(4); JSmax = v_get(3); JJSm = v_get(3);
	for (i=0; i<3; i++)  {  JSmax->ve[i]=-9999; ks->ve[i]=visuservParams.lambda[i];  Eta->ve[i]=0.0; 
		for (j=0; j<3; j++)  {JS->me[i][j]= J->me[i][j]*STF->ve[i]*STF->ve[j]; JSA->me[i][j]= J->me[i][j]*STF->ve[i]*SATF->ve[j]; 
	             if (JSmax->ve[i] <JS->me[i][j]) {JJSm->ve[i]==j; JSmax->ve[i]=JS->me[i][j];}}
			     }
      
	// Sliding mode condition -> S21 sign 
	if (STF->ve[1]*STF->ve[2]>0) { printf("Cor39-> "); goto l2;  	} 

    	if (visuservParams.sm_type== VISUSERV_SLIDING_1_SIGN) {
        
	// k1 and k2 computation under sliding condition ss_dot<0 (k0=cte)
	beta=-(JS->me[0][0]*ks->ve[0]+JS->me[0][1]*ks->ve[1])/(JS->me[0][2]);
	gamma=-(JS->me[1][0]*ks->ve[0]+JS->me[1][1]*ks->ve[1])/(JS->me[1][2]);
	
	// case I: S20 ++ ==>  max(beta,gamma) < k2 < k1
	if (STF->ve[2]*STF->ve[0]>0 & JS->me[1][2]!=0) {

	if (ks->ve[1]>=max(gamma,beta) & max(gamma,beta)>0) {printf("Cor11-> "); ks->ve[2]=0.5*(ks->ve[1]+max(gamma,beta)); 	}
	else if (ks->ve[1]>max(gamma,beta) & max(gamma,beta)<=0) {printf("Cor12-> "); ks->ve[2]=0.95*ks->ve[1]; 	}
	else if (ks->ve[1]<max(gamma,beta) & max(gamma,beta)<= q_dot_max) {printf("Cor13-> ");ks->ve[1]=1.2*max(gamma,beta);ks->ve[2]=0.9*ks->ve[1]; 	}
	else {  printf("Cor19-> ");   /*goto l2;*/  	}
					}
	// case II: S20 -- ==> min(beta,k1) < k2 < gamma 
	if (STF->ve[2]*STF->ve[0]<0 & JS->me[1][2]!=0) {

	if (gamma>=0 & gamma<ks->ve[1] & beta>0) { printf("Cor21-> "); 	ks->ve[2]=0.5*(min(ks->ve[1],beta)+ gamma); }
	else if (beta>0 & gamma<0) {	printf("Cor22-> "); 	ks->ve[2]=0.95*ks->ve[1]; 	}
	else if (beta>0 & gamma>0 & gamma>min(ks->ve[1],beta)) { printf("Cor23-> ");   ks->ve[1]=1.1*gamma; ks->ve[2]=0.95*ks->ve[1]; 	}
	else { printf("Cor29-> "); 	/*goto l2; */ }
					}
		Eta = mv_mlt(JS,ks, VNULL);

	// sliding mode controller TYPE 1 : q_dot = - [ J^-1 * C * ds/dt + K * Sign(TaskFunction)] 
        for (k=0; k<3; k++) {visuservParams.lambda[k]=ks->ve[k];}
	K=GetLambdaMAT(visuservParams.lambda, 3, MNULL);
	qpoint = v_get(3); qpoint1 = v_get(3); qpoint2 = v_get(3);
	//qpoint1 = mv_mlt(m_mlt(Ji,C,MNULL),ds, VNULL);	v_add(qpoint1 ,qpoint2 ,qpoint);
	qpoint = mv_mlt(K,STF, VNULL);
							}

	 else if (visuservParams.sm_type== VISUSERV_SLIDING_2_SAT) {

		ks->ve[2]=-0.9*(ks->ve[1]*SATF->ve[1])/SATF->ve[2];
		printf("Cor49-> %.1f ",ks->ve[2]);
		Eta = mv_mlt(JSA,ks, VNULL);

	// sliding mode controller TYPE 2 : q_dot = - [ J^-1 * C * ds/dt + K * Sat(TaskFunction)] 
        for (k=0; k<3; k++) {visuservParams.lambda[k]=ks->ve[k];}
	K=GetLambdaMAT(visuservParams.lambda, 3, MNULL);
	qpoint = v_get(3); qpoint1 = v_get(3); qpoint2 = v_get(3);
	//qpoint1 = mv_mlt(m_mlt(Ji,C,MNULL),ds, VNULL);	v_add(qpoint1 ,qpoint2 ,qpoint);
	qpoint = mv_mlt(K,SATF, VNULL);

							}

/* 	// case I: S20 ++ ==>  max(beta,gamma) < k2 < k1
	if (STF->ve[2]*STF->ve[0]>0) {  
		if (max(beta,gamma)>=ks->ve[1])
			{  cor=11; ks->ve[1]=1.01*max(beta,gamma); ks->ve[2]=(ks->ve[1]+max(beta,gamma))/2; }
		else if (max(beta,gamma)>0)   
			{  cor=12; ks->ve[2]=(ks->ve[1]+max(beta,gamma))/2; }
		else 	{  cor=13; ks->ve[2]=0.9*ks->ve[1]; }
					}

	// case II: S20 -- ==> min(beta,k1) < k2 < gamma 
	if (STF->ve[2]*STF->ve[0]<0) {
		if (min(beta,ks->ve[1])>gamma & min(beta,ks->ve[1])>0) 	
			{  cor=21; ks->ve[2]=0.9*min(ks->ve[1],beta);}
		else if (min(beta,ks->ve[1])<gamma & gamma>0) 	
			{  cor=22; ks->ve[1]=1.1*gamma; ks->ve[2]=0.9*ks->ve[1]; }	
	 	else goto l2;//else   ks->ve[2]=0.9*ks->ve[1];			
			---------------------------------------------------------

// Simplex minimization to get Eta:
	for (k=0; k<3; k++)  { Eps->ve[k]=visuservParams.lambda[k]; Cost->ve[k]=fabs(JS->me[0][k])+fabs(JS->me[1][k])+fabs(JS->me[2][k]);}
	Simp=Simplex(Eps, Cost, JS, VNULL); 	for (k=0; k<3; k++)  { Eta->ve[k]=Simp->ve[k]; } */	

	// Sliding condition : K = SUM J-1(i,j) * S_i * S_j * Eta_i :
	// l1:	if (js>=20) { printf("js=%d",js); goto l2;}
	//mv_mlt(JS,Eta,ks); 	for (k=0; k<3; k++)  {state->KSM[k]=ks->ve[k]; printf("k= %.2f,",ks->ve[k]);}
	//if(ks->ve[1]<=0 || ks->ve[1]<=0) goto l2; else for (k=0; k<3; k++) {visuservParams.lambda[k]=ks->ve[k];}

	/* check K_i signs:
	if(ks->ve[1]<=0 & JSmax->ve[1]>0) {Eta->ve[int(JJSm->ve[1])]=1.1*Eta->ve[int(JJSm->ve[1])]; js++;  goto l1; }
	else if(ks->ve[2]<=0 & JSmax->ve[2]>0) {Eta->ve[int(JJSm->ve[2])]=1.1*Eta->ve[int(JJSm->ve[2])]; js++; goto l1;}
	else if(ks->ve[1]<=0 || ks->ve[1]<=0)  goto l2;*/
			      
	/* control gain under sliding condition ss_dot<0 :
	beta=-(JS->me[0][0]*visuservParams.lambda[0]+JS->me[0][1]*visuservParams.lambda[1])/(JS->me[0][2]);
	gamma=-(JS->me[1][0]*visuservParams.lambda[0]+JS->me[1][1]*visuservParams.lambda[1])/(JS->me[1][2]);

     if (STF->ve[2]*STF->ve[1]>0) {
   	  cor=0; printf("|Cc:%d,TF32=%.2f|",cor,TF->ve[1]*TF->ve[2]);
	// use classic control
	
			return OK;	}
     else { 
	// case I : S13 -  S23 -
	if (STF->ve[2]*STF->ve[0]<0 & STF->ve[1]*STF->ve[2]<0) {  cor=1;
	    if (max(beta,gamma)<visuservParams.lambda[1]) { 
	   	  if (max(beta,gamma)>0) visuservParams.lambda[2]=(max(beta,gamma)+visuservParams.lambda[1])/2; 
		  else visuservParams.lambda[2]=0.9*min(visuservParams.lambda[1],gamma);}
	    else goto l2;  					}
	// case II: S13 -  S23 +
	else if (STF->ve[2]*STF->ve[0]<0 & STF->ve[1]*STF->ve[2]>0) {  cor=2;
	    if (min(visuservParams.lambda[1],gamma)>beta) {
		 if (beta>0) visuservParams.lambda[2]=(min(visuservParams.lambda[1],gamma)+beta)/2;
		 else if (beta<0 & gamma>0) visuservParams.lambda[2]=0.9*min(visuservParams.lambda[1],gamma);
	    	 else goto l2;}
	    else goto l2; 					    }
	// case III : S13 +  S23 -
	else if (STF->ve[2]*STF->ve[0]>0 & STF->ve[1]*STF->ve[2]<0) {cor=3;
	     if (min(visuservParams.lambda[1],beta)>gamma){ 
	     	if (beta>0 & gamma>0) visuservParams.lambda[2]=(min(visuservParams.lambda[1],beta)+gamma)/2; 
	     	else if (gamma<0 & beta>0) visuservParams.lambda[2]=0.9*min(visuservParams.lambda[1],beta);
		    	 else goto l2;}
             else goto l2;       					}
	// case IV : S13 +  S23 +
	else 	{ cor=4; if  (min(min(visuservParams.lambda[1],beta),gamma)>0) 
			visuservParams.lambda[2]=min(min(visuservParams.lambda[1],beta),gamma)/2; 
		   else goto l2;
		}
 	     }
        for (k=0; k<3; k++) {state->KSM[k]=visuservParams.lambda[k];printf(" %.2f,",visuservParams.lambda[k]);}*/

		
	// compute lyapunov function derivate sigma*sigma_dot :
	TFP = v_get(3); TFP1 = v_get(3);TFP2 = v_get(3);SSP = v_get(3);
	mv_mlt(J,qpoint,TFP); //mv_mlt(C,ds,TFP2); v_add(TFP1,TFP2,TFP);	
	for (k=0; k<3; k++)  {state->TFP[k]= TFP->ve[k];SSP->ve[k]=(TF->ve[k])*(TFP->ve[k]); state->KSM[k]=Eta->ve[k];}

	printf(" Eta:%.1f/%.1f/%.1f| k:%.3f/%.3f/%.3f|V= %.3f|W= %.3f|P= %.3f|",Eta->ve[0],Eta->ve[1],Eta->ve[2],ks->ve[0],ks->ve[1],ks->ve[2],qpoint->ve[0],qpoint->ve[1],qpoint->ve[2]);
	
	// Software limits
	if (qpoint->ve[0]>0.08) {qpoint->ve[0] = 0.08;/*printf ("v+");*/} 
	if (qpoint->ve[0]<= 0) {qpoint->ve[0] = -0.08;/*printf ("v-");*/} 
	if (qpoint->ve[1]>0.25) {qpoint->ve[1] = 0.25;/*printf ("w+");*/} 
	if (qpoint->ve[1]<-0.25) {qpoint->ve[1] = -0.25;/*printf ("w-");*/} 
	if (qpoint->ve[2]>0.25) {qpoint->ve[2] = 0.25; /*printf ("p+");*/} 
	if (qpoint->ve[2]<-0.25) {qpoint->ve[2] = -0.25;/*printf ("p-");*/} 
	
	dot_q->vr=qpoint->ve[0]; 	dot_q->wr=qpoint->ve[1];    dot_q->p=qpoint->ve[2];   dot_q->t=ks->ve[1];

	// use classical controller
	if (js<0) { l2: VSCParams.lambda[0]=0.00004;  VSCParams.lambda[1]=0.00004;  VSCParams.lambda[2]=0.004;  		
			VSCParams.vs_mode=visuservParams.vs_mode;
	vsController(pan,pan,C,s_ref,s,s_old,VSCParams,qp_old,dot_q,state);  return OK;}

	/*garbage*/
	m_free(K);m_free(L);m_free(Ji); m_free(J); m_free(JS);
	v_free(TF); v_free(STF); v_free(TFP); v_free(TFP1);v_free(TFP2);v_free(ds); v_free(SSP);v_free(Simp);
	v_free(qpoint); v_free(qpoint1); v_free(qpoint2);v_free(Eta);v_free(ks);v_free(Eps);v_free(Cost);v_free(JSmax);v_free(JJSm);
	
	return OK;
}

/***************************************************************************************/
/********************************** Simplex - a reverifier les indices *****************/
VEC* Simplex(VEC *Epsilon , VEC *Cost, MAT *Cnts, VEC *Eta){

 int i,j,k,ip,jp,cmp=1;
 double MinEps=999999,Pivot=-999999,Sif;
 MAT *AM= MNULL;  // augmented matrix 
  
    // init and fill augmented matrix
       AM = m_get(4,7); Eta=v_get(4);
  //     printf("Simplex start AM = \n");
       for (i=0; i< 3;i++ ){  
           for (j=0; j< 3;j++ ){ AM->me[i][j] = Cnts->me[j][i];  }      // transposed constraintes variables 
		AM->me[i][i+3] = 1.0;					//slask variables
		AM->me[i][6] = Cost->ve[i];				//Cost function factors
		AM->me[3][i] = -Epsilon->ve[i];				//Basic variables
		 	}
       //for (i=0; i< 4;i++ ){  for (j=0; j< 7;j++ ){ printf(" %lg  ",AM->me[i][j]);  } printf(" \n"); }   //printf("Sim

   /*  search pivot */
  l1:  for (k=0; k< 3;k++ ){ if(AM->me[3][k]<MinEps) {MinEps = AM->me[3][k];  jp=k; }	}
         if (MinEps >=0)  { printf(" min = %.2f ==> ",AM->me[3][k]); goto l2; } 
  	 //  else printf("\n Iteration n° %d min= %.2f at colum %d -->",cmp,MinEps,jp);
		
       for (k=0; k< 3;k++ ){ if(AM->me[k][jp]>Pivot&AM->me[k][jp]!=0) {Pivot= AM->me[k][jp];	  ip=k;}	 } 
	//printf(" Piv= %.2f at line %d\n",Pivot,ip);

	// pivoter la matrice AM
	for (i=0; i< 4 ; i++ )  {   if (i!=ip)  { Sif=AM->me[i][jp];  //printf(" Sif= %.2f \n",Sif);
					for (j=0; j< 7;j++ ){AM->me[i][j] = AM->me[i][j]-Sif*AM->me[ip][j]/Pivot;} 
			    			}
				}
	for (j=0; j< 7;j++ ){ AM->me[ip][j] = AM->me[ip][j]/Pivot;}
	//printf("New AM = \n");
        //for (i=0; i< 4;i++ ){  for (j=0; j< 7;j++ ){ printf(" %.2f  ",AM->me[i][j]);  } printf(" \n"); }   
	
        // Simplex temporary sollution
        //for (k=3; k< 6;k++ ){ if (AM->me[3][k]<0)  { printf(" Negative Eta %lg ==> ",AM->me[3][k]); goto l2;  } }
        for (k=0; k< 3;k++ ) {Eta->ve[k]=AM->me[3][k+3]; } 
        MinEps = 999; Pivot=-999;cmp++;  
	goto l1; 

 l2:	for (k=0; k< 3;k++ ) {printf("Eta%d= %lg |",k+1,Eta->ve[k]);}
	Eta->ve[3]=1.0;printf("Simplex END, cmp=%d \n",cmp);
			 
	return Eta;
}

/************************ VS DYNAMIC : Mobile target (ds/dt!=0) ***** from here --> not used yet *************/
/****													****/
STATUS vs2Controller(double pan,MAT *C,ICU_VISUAL_DATA s_ref, ICU_VISUAL_DATA s,ICU_VISUAL_DATA s_old, VISUSERV_VISUAL_SERVOING_PARAMS_STR visuservParams, VISUSERV_ROBOT_CONTROLS_STR *dot_q, VISUSERV_STATE_STR *state){
	
	VEC *qpoint,*qpoint1,*qpoint2,*taskFunc=VNULL,*ds=VNULL,*ds1=VNULL,*ds2=VNULL;   // ds = s_ols - s  
	MAT *J=MNULL,*L=MNULL,*Ji=MNULL, *mLambda=MNULL;    
	int i,k,j;
	static int  d1=0,d2=0;
	double Lbda[3], dt=0.04;

	// L type display	
	if (visuservParams.vs_mode==VISUSERV_CONSTANT_INTER_MTRX) printf("Dst|");
	else if (visuservParams.vs_mode==VISUSERV_BOTH) printf("Dbo|"); 
	else printf("Dva|"); 	
	
	//Get the taskFunction 
	if (GetVisuservTaskFunc(s_ref,s,C,&taskFunc) != OK) return ERROR;
	printf("DYTF: %.3f-> ",v_norm2(taskFunc));
	for (k=0; k<3; k++)  {state->TF[k]= taskFunc->ve[k]; printf("%.3f |",taskFunc->ve[k]);}
	        	
	/* if we are close to the goal or not*/
	if (v_norm2(taskFunc)<visuservParams.epsi) { 
	  dot_q->vr = 0.0; dot_q->wr = 0.0;  dot_q->p = 0.0; 	state->onGoal=1;
	 	if(d1==0) {printf("<--- Robot onGoal & TF= %f --->",v_norm2(taskFunc));d1=1;}
          	v_free(taskFunc);  return OK; } 
	else {state->onGoal=0;d1=0;}
	
	/* Compute -ds/dt = s_old - s /dt (en metres): old*/
        ds = v_get(8); for (i=0, j=0; i<4; i++, j+=2 ){
	ds->ve[j]      = (double)(focal*(s_old.x[i]-s.x[i])/(alpha_u*dt));    
	ds->ve[j+1]    = (double)(focal*(s_old.y[i]-s.y[i])/(alpha_v*dt));
	//printf("DST: %.2f , %.2f | ",ds->ve[j],ds->ve[j+1]);  
	}

	/* Compute -ds/dt = L*J*q_dot + s_old - s /dt (en metres):
        ds = v_get(8); ds1 = v_get(8);ds2 = v_get(8);qp_old = v_get(3);
	qp_old->ve[1] = oldSpeeds.vr; qp_old->ve[1] = oldSpeeds.wr; qp_old->ve[1] = oldSpeeds.p; 
	for (i=0, j=0; i<4; i++, j+=2 ){ds1->ve[j]      = (double)(focal*(s_old.x[i]-s.x[i])/(alpha_u*dt));    
					ds1->ve[j+1]    = (double)(focal*(s_old.y[i]-s.y[i])/(alpha_v*dt));	}
	L= GetVisualDataInteractionMatrix(s,MNULL);   GetRobotJacobian(pan,&J); 
 	ds2 = mv_mlt(m_mlt(L,J,MNULL),qp_old,VNULL);	v_add(ds1,ds2,ds);
	printf(" ds= %lg , %lg|",ds->ve[0],ds->ve[1]);

	/*compute the jacobian*/
	GetInverseRobotJacobian(pan,&Ji ); 
	//for (i = 0; i < 3 ; i++){for (j=0; j < 3; j++) printf("| %.2f ",J->me[i][j]);}	printf("\n");
		
	// Lambda tuning and Lambda matrix computation :
	Lbda[0]=visuservParams.lambda[0];     Lbda[1]=visuservParams.lambda[1];    Lbda[2]=visuservParams.lambda[2];	
	//if(fabs(pan)>0.02){Lbda[0]=(double)(fabs(sin(pan))*visuservParams.lambda[0]);printf("@P");} 
	mLambda=GetLambdaMAT(Lbda, 3, MNULL);	//for (i = 0; i < 3 ; i++){ printf(" %lg|", mLambda->me[i][i]);} 
	
	//Compute dot_q = dot_q1 + dot_q2 = [-J^-1 * LAMBDA * C * (s -s_ref)] + [-J^-1 * C * (s -s_old)/dt]	
	qpoint = v_get(3); qpoint1 = v_get(3); qpoint2 = v_get(3);
	qpoint1 = mv_mlt(m_mlt(Ji,mLambda,MNULL),taskFunc, VNULL );	qpoint2 = mv_mlt(m_mlt(Ji,C,MNULL ),ds,VNULL);	
	v_add(qpoint1 ,qpoint2 ,qpoint);
        printf("V= %.3f ,W= %.3f, %.3f|P= %.3f |",qpoint->ve[0],qpoint1->ve[1],qpoint2->ve[1],qpoint->ve[2]);

	// Software limits
//	if (qpoint->ve[0]>0.0) {qpoint->ve[0] = 0.05;printf (" V+");} 
//	if (qpoint->ve[0]<= 0) {qpoint->ve[0] = 0.0;printf (" V-");} 
	if (qpoint->ve[1]>0.25) {qpoint->ve[1] = 0.25;printf (" W+");} 
	if (qpoint->ve[1]<-0.25) {qpoint->ve[1] = -0.25;printf (" W-");} 
	if (qpoint->ve[2]>0.25) {qpoint->ve[2] = 0.25; printf (" P+");} 
	if (qpoint->ve[2]<-0.25) {qpoint->ve[2] = -0.25;printf (" P-");} 
	printf("av affec");	 
	dot_q->vr  = qpoint->ve[0]; 	dot_q->wr = qpoint->ve[1];    dot_q->p  = qpoint->ve[2];
	printf("ap affec av free");	 

	/*garbage*/
	m_free(Ji);	m_free(J); v_free(taskFunc); v_free(ds);
	m_free(mLambda); v_free(qpoint); v_free(qpoint1); v_free(qpoint2);
	v_free(ds); v_free(ds1); v_free(ds2);
	printf("ap free");	 

	return OK;
}

/*************************************** Read Robot Config ************************************************/
STATUS visuservReadRobotConfig(VISUSERV_ROBOT_CONFIGURATION_STR *q){

	VISUSERV_ROBOT_CONFIGURATION_STR  	robot;
	PLATINE_STATES   			effector;
 		
	int offset;
	
   /*recuperation de l'etat de l'effecteur*/
   /*offset = (char *)&effectorPoster->stateRad - (char *)effectorPoster;*/
 
if (posterFind(SDI_F->postersName.effectorSpeedPosterName,(POSTER_ID*)&SDI_F->postersID.effectorSpeedPosterID)!=OK){
     printf( "visuservSetTarget:  poster %s not found \n",SDI_F->postersName.effectorSpeedPosterName);
    return ETHER;  }

if (posterRead((POSTER_ID*)&SDI_F->postersID.effectorSpeedPosterID, offset,(char *)&effector,sizeof(effector))!= sizeof(effector)){ 
     printf(" visuservSetTarget: can't read poster\n");
    return ETHER; }

   //printf(" Pan =  %.2f |Tilt = %.2f | \n",PTUstate.stateDeg.pos.pan,PTUstate.stateDeg.pos.tilt); 

   /*recuperation de l'etat du robot
   if (posterFind(SDI_F->postersName.robotSpeedPosterName,(POSTER_ID*)&SDI_F->postersID.robotSpeedPosterID)!=OK){
     printf( "visuservSetTarget:  poster %s not found \n",SDI_F->postersName.robotSpeedPosterName);
    return ETHER;  }

   if (posterRead((POSTER_ID*)&SDI_F->postersID.robotSpeedPosterID, offset,(char *)&robot,sizeof(robot))!= sizeof(robot)){ 
     printf(" visuservSetTarget: can't read poster\n");
    return ETHER; }*/

	
	printf(" visuservReadRobotConfig: pan= %g \n",effector.stateDeg.pos.pan);
		
	/* visuServPrint("set robot config \n");*/
	if (q!=NULL){
	//	q->x     = robot.x;
	//	q->y     = robot.y;
	//	q->theta = robot.theta;
		q->pp 	 = effector.stateDeg.pos.pan;
	//	q->l 	 = SDI_F->servoingRef.q.l; 
		}		
	return OK;
}

/******************************* Read Visual Features *************** not used yet ************************/
/******************************* Read Visual Features *************** not used yet ************************/
/******************************* Read Visual Features *************** not used yet ************************/
/******************************* Read Visual Features *************** not used yet ************************/
/******************************* Read Visual Features *************** not used yet ************************/
STATUS visuservReadVisualFeatures(ICU_VISUAL_DATA *Features){

   	//  VISUSERV_OCCLUDING_STR occluding;

  int offset=0, i,j;  
    

    // read visual data from ICU 
if(posterFind(SDI_F->postersName.visualDataPosterName,(POSTER_ID*)&SDI_F->postersID.visualDataPosterID)==ERROR) {
    printf ("ReadVisualFeatures: poster %s not found\n",SDI_F->postersName.visualDataPosterName);
    SDI_F->postersID.visualDataPosterID=-1;
    return END;  }

   	printf("potsre ID : %d", SDI_F->postersID.visualDataPosterID);
  
// Read references from poster 
  if (posterRead((POSTER_ID*)SDI_F->postersID.visualDataPosterID,0,(void *) &Features,sizeof(Features))==ERROR) 
  { printf ("ReadVisualFeatures: poster read error \n");
    SDI_F->postersID.visualDataPosterID=-1;
    return END;  }

	printf(" xo = %.2f | yo = %.2f | zo= %.2f  \n",Features->x[0],Features->y[0],Features->z[0]);
 	printf(" x1 = %.2f | y1 = %.2f | z1= %.2f  \n",Features->x[1],Features->y[1],Features->z[1]);
  	printf(" x2 = %.2f | y2 = %.2f | z2= %.2f  \n",Features->x[2],Features->y[2],Features->z[2]);
  	printf(" x3 = %.2f | y3 = %.2f | z3= %.2f  \n",Features->x[3],Features->y[3],Features->z[3]);
  	printf("------------Data ---------- \n"); 
   /* Z[i] Computation
   	for (i = 0; i < target.nbObjects; i++){
	// Features->x[i] = (double)(target.pobjects[i].Ucg - VISUSERV_U0) / VISUSERV_KU; 
	//Features->y[i] = (double)(target.pobjects[i].Vcg - VISUSERV_V0) / VISUSERV_KV;

	Features->x[i] =(double)PIX_UtoMETRIC(target.pobjects[i].Ucg - VISUSERV_U0); 
	Features->y[i] =(double)PIX_VtoMETRIC(target.pobjects[i].Vcg - VISUSERV_V0);
	
	//depth processing
	diam=PIXtoMETRIC((4.0*target.pobjects[i].Surface)/(M_PI*((double)target.pobjects[i].Umax-
	target.pobjects[i].Umin)));
	di=(double)VS_SQUARE(PIX_VtoMETRIC(target.pobjects[i].Ucg-VISUSERV_U0))+(double)VS_SQUARE(PIX_VtoMETRIC
	(target.pobjects[i].Vcg-VISUSERV_V0));
	zi   = sqrt((double) VS_SQUARE(VISUSERV_FOCUS) + di);  //VS_SQUARE(VISUSERV_TPIX)
	Features->z[i]=(double)(SDI_F->visualServoingParams.diameter*zi)/(diam);
					}*/
	return OK;
}

/*************************************** Read SPEED ******************************* not used yet *********/
STATUS visuservReadRobotSpeed(VISUSERV_ROBOT_CONTROLS_STR *dot_q,int *report){

	VISUSERV_ROBOT_CONFIGURATION_STR     robotSpeed;
	VISUSERV_PLATINE_CONFIGURATION_STR   effectorSpeed;
	int offset;

	// read robotSpeed Poster
	offset =0;
	if (posterFind(SDI_F->postersName.robotSpeedPosterName, (POSTER_ID *)&SDI_F->postersID.robotSpeedPosterID) == ERROR) {
    		SDI_F->postersID.robotSpeedPosterID = -1;
    		*report = S_visuserv_POSTER_ERROR;
    		printf ("ReadRobotSpeed : poster %s robot not found\n",SDI_F->postersName.visualDataPosterName);
  	return ETHER;  }
  
	 if (posterRead((POSTER_ID*)SDI_F->postersID.robotSpeedPosterID,0,(void *) &robotSpeed,sizeof(ICU_VISUAL_DATA))==ERROR) 
	  { SDI_F->postersID.robotSpeedPosterID = -1;
	    *report = S_visuserv_POSTER_ERROR;
	    return ETHER;  }

	// read effectorSpeed Poster 
	effectorSpeed.pan=0.0;
	if (posterFind(SDI_F->postersName.effectorSpeedPosterName, (POSTER_ID *)&SDI_F->postersID.robotSpeedPosterID) == ERROR) {
    		SDI_F->postersID.robotSpeedPosterID = -1;
    		*report = S_visuserv_POSTER_ERROR;
    		printf ("ReadRobotSpeed : poster %s effector not found\n",SDI_F->postersName.visualDataPosterName);
  	return ETHER;  }
  
       if (posterRead((POSTER_ID*)SDI_F->postersID.robotSpeedPosterID,0,(void *) &effectorSpeed,sizeof(ICU_VISUAL_DATA))==ERROR) 
	  	{ SDI_F->postersID.robotSpeedPosterID = -1;
	    	*report = S_visuserv_POSTER_ERROR;
	    	return ETHER;  }
		
	if (dot_q!=NULL){
		dot_q->vr    = robotSpeed.vr;
		dot_q->wr    = robotSpeed.wr;
		dot_q->p    = effectorSpeed.pan;
	}

	//store robot velocities 
//	SDI_F->servoingRef.dot_q=*(dot_q);
	printf(" visuservReadRobotSpeed [#%d] =====> dot_q=(%g, %g, %g)\n\n",SDI_F->servoingRef.numRef,
                dot_q->vr,RAD2DEG(dot_q->wr),RAD2DEG(dot_q->p));	
	return OK;
}

/************************************ Update SPEED ********************** not used yet ***************/
STATUS visuservUpdateSpeed(VISUSERV_ROBOT_CONTROLS_STR dot_q,  int num_ref, int *bilan){
	
	GENPOS_CART_SPEED robotRef;
	VISUSERV_PLATINE_CONTROL_STR effectorRef;
  	static struct timeval endTime;  
	
	// Controle variables Saturation 
	if (fabs(dot_q.vr)   > SDI_F->robotParams.vMax)   dot_q.vr    = Sign(dot_q.vr)*SDI_F->robotParams.vMax;
	if (fabs(dot_q.wr)   > SDI_F->robotParams.wMax)   dot_q.wr    = Sign(dot_q.wr)*SDI_F->robotParams.wMax;
	if (fabs(dot_q.p)   > SDI_F->robotParams.pVMax)  dot_q.p    = Sign(dot_q.p)*SDI_F->robotParams.pVMax;
	
	printf("visuservUpdateSpeed [#%d] up:%d :   vlin= %g (m/s) ; w=%g (deg/s); w_pl=%g (deg/s)  (write to poster=>send to robots...)\n",num_ref,VISUSERV_SPEED_UPDATE_PERIOD, dot_q.vr,RAD2DEG(dot_q.wr),RAD2DEG(dot_q.p));

	//  Write Controle variables in poster 
	robotRef.updatePeriod = VISUSERV_SPEED_UPDATE_PERIOD;	  
	robotRef.v = dot_q.vr;	                                
	robotRef.w = dot_q.wr;		                                  
	robotRef.vmax = SDI_F->robotParams.vMax;             
	robotRef.wmax = SDI_F->robotParams.wMax;             
	robotRef.linAccelMax = SDI_F->robotParams.linAccMax; 
	robotRef.angAccelMax = SDI_F->robotParams.angAccMax;	
	robotRef.numRef = (num_ref)++;                 
	
	/* if (visuservSpeedRefSpeedRefPosterWrite(VISUSERV_SPEEDREF_POSTER_ID, &robotRef)!=OK){
		printf("visuservUpdateSpeed : an error occur when writing in SpeedRef poster  \n");
		*bilan=S_visuserv_POSTER_ERROR;
		return ERROR; }

	effectorRef.pan = dot_q.p;
	effectorRef.tilt= 0.;
	
	if (visuservEffectorRefEffectorRefPosterWrite(VISUSERV_EFFECTORREF_POSTER_ID, &effectorRef)!=OK){
		printf("visuservUpdateSpeed : an error occur when writing in EffectorRef poster  \n");
		*bilan=S_visuserv_POSTER_ERROR;
		return ERROR; }

	/** @todo use mean to compute samplingTime?	 */
  if (SDI_F->servoingRef.samplingTime==0){
    //gettimeofday(&previousTime,VISUSERV_TIMEZONE);
		SDI_F->servoingRef.q.l =0.;
		SDI_F->servoingRef.numRef=0;
    		previousQ=SDI_F->servoingRef.q;
 
    gettimeofday (&endTime , VISUSERV_TIMEZONE ) ;
    //SDI_F->servoingRef.samplingTime =visuservGetElapsedTime(&previousTime,&endTime);
	}else{
    gettimeofday (&endTime , VISUSERV_TIMEZONE ) ;
    //SDI_F->servoingRef.samplingTime =visuservGetElapsedTime(&previousTime,&endTime);    
    /*SDI_F->servoingRef.q.abscisse_curv += fabs(dot_q.v)*SDI_F->servoingRef.samplingTime*1.0e-3;*/
    SDI_F->servoingRef.q.l += sqrt((previousQ.x-SDI_F->servoingRef.q.x)*(previousQ.x-SDI_F->servoingRef.q.x)+
                                   (previousQ.y-SDI_F->servoingRef.q.y)*(previousQ.y-SDI_F->servoingRef.q.y)); }  
  
	printf("visuservUpdateSpeed [#%d]  l=%+12.6lg (m);  Ts=%g (ms) \t (%g ; %g)->(%g ; %g)\n ",
	SDI_F->servoingRef.numRef,SDI_F->servoingRef.q.l, 
	SDI_F->servoingRef.samplingTime,previousQ.x,previousQ.y,SDI_F->servoingRef.q.x,SDI_F->servoingRef.q.y);
  
  SDI_F->servoingRef.numRef++;
  previousTime=endTime;
  previousQ=SDI_F->servoingRef.q;  
  return OK;
}
 
/************************************** visual Data estimation ******************* not used yet *****************

STATUS visuServEstimVisualFeatures(VISUSERV_ROBOT_CONFIGURATION_STR qk, ICU_VISUAL_DATA sk, VISUSERV_ROBOT_CONTROLS_STR dot_qk, double h, ICU_VISUAL_DATA *s_estim, int *report){

	int status=VS_SUCCESS,i;
	//VS_ODE_ESIMSZ_PARAMS_STR estimSZ_params;
	VS_SCREW_STR screw_k;

#ifndef VISUSERV_FORCE_ESTIMATOR
	if(SDI_F->visualServoingParams.useEstimator<VS_ODE_SCHEME_MIN){
		visuServPrintWarning("visuServEstimVisualFeatures: visual feature estimator disabled!\n");
		return OK;
	}
#endif

	if ((SDI_F->state.targetRefIsDefined==FALSE)||(SDI_F->visualDataRef.nbSpots==0)){
		visuServPrintWarning("visuServEstimVisualFeatures: could not use visual feature estimator until target ref is defined!\n");
		return OK;
	}

	if (sk.nbSpots==0){
		printf("visuServEstimVisualFeatures: no enought previous visual data to try estimate it...\n");
		*report=S_visuServ_ESTIMATOR_ERROR;
		return ERROR;
	}

	status=vsGetCameraScrew(qk.theta_pl,VISUSERV_Dx,VISUSERV_Cx,VISUSERV_Cy,dot_qk,&(screw_k));
	VS_IF_ERROR_ERRNO("visuServStepScrew(): cannot get screw",status);

 
#ifndef VISUSERV_FORCE_ESTIMATOR	
	switch(SDI_F->visualServoingParams.useEstimator){
		default:
			VS_WARNING("visuServEstimVisualFeatures(): unknow or bad ODE scheme given, use RK4 as default\n");
		case VS_ODE_SCHEME_RK4:       // 4th order Runge-Kutta scheme
			status=visuServRK4(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
			break;
		case VS_ODE_SCHEME_ABM:       // Adams Bashforth Moulton scheme  
			status=visuServABM(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
			break;
		case VS_ODE_SCHEME_AB:       // Adams Bashforth scheme  
			status=visuServAB(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
			break;
		case VS_ODE_SCHEME_BDF:       // BDF scheme  
			status=visuServBDF(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
			break;
		case VS_ODE_SCHEME_EULER:       // BDF scheme  
			status=visuServEuler(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
			break;

	}
#else
	printf("----------------------------------------------------------------------\n");
	printf("\tEuler SCHEME\n");			
	status=visuServEuler(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
	for (i = 0; i < s_estim->nbSpots ; i++){
		VS_PRINT("REAL? s_%d (%lg, %lg, %lg)\n", i,
						 sk.x[i]/VISUSERV_TPIX+VISUSERV_U0,sk.y[i]/VISUSERV_TPIX+VISUSERV_V0,sk.z[i]);
		VS_PRINT("ESTIM s_%d (%lg, %lg, %lg)\n", i,
						 s_estim->x[i]/VISUSERV_TPIX+VISUSERV_U0,s_estim->y[i]/VISUSERV_TPIX+VISUSERV_V0,s_estim->z[i]);
		VS_PRINT("ERROR s_%d (%lg, %lg, %lg)\n", i,
						 (sk.x[i]-s_estim->x[i])/VISUSERV_TPIX,(sk.y[i]-s_estim->y[i])/VISUSERV_TPIX,(sk.z[i]-s_estim->z[i]));
	}

	printf("----------------------------------------------------------------------\n");
	printf("\tRK4 SCHEME\n");			
	status=visuServRK4(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
	for (i = 0; i < s_estim->nbSpots ; i++){
		VS_PRINT("REAL? s_%d (%lg, %lg, %lg)\n", i,
						 sk.x[i]/VISUSERV_TPIX+VISUSERV_U0,sk.y[i]/VISUSERV_TPIX+VISUSERV_V0,sk.z[i]);
		VS_PRINT("ESTIM s_%d (%lg, %lg, %lg)\n", i,
						 s_estim->x[i]/VISUSERV_TPIX+VISUSERV_U0,s_estim->y[i]/VISUSERV_TPIX+VISUSERV_V0,s_estim->z[i]);
		VS_PRINT("ERROR s_%d (%lg, %lg, %lg)\n", i,
						 (sk.x[i]-s_estim->x[i])/VISUSERV_TPIX,(sk.y[i]-s_estim->y[i])/VISUSERV_TPIX,(sk.z[i]-s_estim->z[i]));
	}

	printf("----------------------------------------------------------------------\n");
	printf("\tABM SCHEME\n");			
	status=visuServABM(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
	for (i = 0; i < s_estim->nbSpots ; i++){
		VS_PRINT("REAL? s_%d (%lg, %lg, %lg)\n", i,
						 sk.x[i]/VISUSERV_TPIX+  VISUSERV_U0,sk.y[i]/VISUSERV_TPIX+ VISUSERV_V0,sk.z[i]);
		VS_PRINT("ESTIM s_%d (%lg, %lg, %lg)\n", i,
						 s_estim->x[i]/VISUSERV_TPIX+ VISUSERV_U0,s_estim->y[i]/VISUSERV_TPIX+ VISUSERV_V0,s_estim->z[i]);
		VS_PRINT("ERROR s_%d (%lg, %lg, %lg)\n", i,
						 (sk.x[i]-s_estim->x[i])/VISUSERV_TPIX,(sk.y[i]-s_estim->y[i])/VISUSERV_TPIX,(sk.z[i]-s_estim->z[i]));
	}

	printf("----------------------------------------------------------------------\n");
	printf("\tBDF SCHEME\n");			
	status=visuServBDF(qk,SDI_F->visualDataRef, SDI_F->visualServoingParams,h,sk,screw_k,s_estim);
	for (i = 0; i < s_estim->nbSpots ; i++){
		VS_PRINT("REAL? s_%d (%lg, %lg, %lg)\n", i,
						 sk.x[i]/VISUSERV_TPIX+  VISUSERV_U0,sk.y[i]/VISUSERV_TPIX+ VISUSERV_V0,sk.z[i]);
		VS_PRINT("ESTIM s_%d (%lg, %lg, %lg)\n", i,
						 s_estim->x[i]/VISUSERV_TPIX+ VISUSERV_U0,s_estim->y[i]/VISUSERV_TPIX+ VISUSERV_V0,s_estim->z[i]);
		VS_PRINT("ERROR s_%d (%lg, %lg, %lg)\n", i,
						 (sk.x[i]-s_estim->x[i])/VISUSERV_TPIX,(sk.y[i]-s_estim->y[i])/VISUSERV_TPIX,(sk.z[i]-s_estim->z[i]));
	}

#endif


	if (status == VS_SUCCESS){
		SDI_F->visualDataEstim=*s_estim;
	}else{
		if (status == VS_WUPDATE){
			// still update... 
			visuServPrintWarning("visuServEstimVisualFeatures: still updating the estimator...\n");
		}else{		
			visuServPrintWarning("visuServEstimVisualFeatures: an error occur while estimate visualFeatures (vs-errno:%d)...\n",status);
			*report=S_visuServ_ESTIMATOR_ERROR;
			return ERROR;
		}			
		if ((*report==S_visuServ_NO_NEW_TARGET)||(*report==S_visuServ_BAD_TARGET)){
			return ERROR;
		}
	} //end if success 

	return OK;
}
*/
/************************************ Merge controllers *************************** not used yet ****************
STATUS visuServMergeControl(VISUSERV_AVOID_ARGS_STR avoid,int filter, VISUSERV_SERVOING_REF_STR *servoingRef){
	VISUSERV_ROBOT_CONTROLS_STR dot_qc; // the new global controller 

	switch(avoid.type){
		case VISUSERV_AVOID_NONE:  //no avoidance 
			dot_qc=servoingRef->dot_q_vs;
			break;
		case VISUSERV_AVOID_OCCLUSION:      //< occlusion avoidance only 
		case VISUSERV_AVOID_OCCLUSION_OBS:        //< occlusion avoidance with obstacle avoidance cost function  
			dot_qc.v    = (1.0-servoingRef->mu_occlu)*servoingRef->dot_q_vs.v    + servoingRef->mu_occlu*servoingRef->dot_q_occlu.v;
			dot_qc.w    = (1.0-servoingRef->mu_occlu)*servoingRef->dot_q_vs.w    + servoingRef->mu_occlu*servoingRef->dot_q_occlu.w;
			dot_qc.w_pl = (1.0-servoingRef->mu_occlu)*servoingRef->dot_q_vs.w_pl + servoingRef->mu_occlu*servoingRef->dot_q_occlu.w_pl;
			break;
		case VISUSERV_AVOID_OCC_OBS:  //< Both occlusion and obstacle avoidance 
       // dot_q = (1-mu_occ)(1-mu_obst)*dot_q_vs+ (1-mu_occ)*mu_obs*dot_q_occ + mu_obs*dot_q_obs	 
			dot_qc.v    = (1.0-servoingRef->mu_occlu)*(1.0-servoingRef->mu_obst)*servoingRef->dot_q_vs.v    + 
				(1.0-servoingRef->mu_obst)*servoingRef->mu_occlu*servoingRef->dot_q_occlu.v    + 
				servoingRef->mu_obst*servoingRef->dot_q_obst.v;
			dot_qc.w    = (1.0-servoingRef->mu_occlu)*(1.0-servoingRef->mu_obst)*servoingRef->dot_q_vs.w    + 
				(1.0-servoingRef->mu_obst)*servoingRef->mu_occlu*servoingRef->dot_q_occlu.w    + 
				servoingRef->mu_obst*servoingRef->dot_q_obst.w;
			dot_qc.w_pl = (1.0-servoingRef->mu_occlu)*(1.0-servoingRef->mu_obst)*servoingRef->dot_q_vs.w_pl + 
				(1.0-servoingRef->mu_obst)*servoingRef->mu_occlu*servoingRef->dot_q_occlu.w_pl    + 
				servoingRef->mu_obst*servoingRef->dot_q_obst.w_pl;
      
      visuServPrintV(VISUSERV_VERBOSITY_VELOCITY,
										 "visuServMerge [#%d]: dot_q_vs  ; vlin= %g ; w=%g ; w_pl=%g \t (no-f) \n",servoingRef->numRef, 
                    servoingRef->dot_q_vs.v, RAD2DEG(servoingRef->dot_q_vs.w), RAD2DEG(servoingRef->dot_q_vs.w_pl));
      visuServPrintV(VISUSERV_VERBOSITY_VELOCITY,
										 "visuServMerge [#%d]: dot_q_occ ; vlin= %g ; w=%g ; w_pl=%g \t (no-f) \n",servoingRef->numRef, 
                    servoingRef->dot_q_occlu.v, RAD2DEG(servoingRef->dot_q_occlu.w), RAD2DEG(servoingRef->dot_q_occlu.w_pl));
      visuServPrintV(VISUSERV_VERBOSITY_VELOCITY,
										 "visuServMerge [#%d]: dot_q_obst  ; vlin= %g ; w=%g ; w_pl=%g \t (no-f) \n",servoingRef->numRef, 
										 servoingRef->dot_q_obst.v, RAD2DEG(servoingRef->dot_q_obst.w), RAD2DEG(servoingRef->dot_q_obst.w_pl));

      break;
			 //unsupported avoidance type 
		case VISUSERV_AVOID_VSRP:
		default:
			printf("visuServMergeControl: unknow avoidance type (%d) or is not yet implemented\n",
												 avoid.type);
			return ERROR;
	}

	visuServPrintV(VISUSERV_VERBOSITY_VELOCITY,
								"visuServMergeControl [%d] ; mu_occlu=%g ; mu_obst=%g ; docc=%g (%g px) ;  dbord=%g (%g px); dobst= %g /ds=%g ; alpha=%g (deg) ; sens=%d\n\t--> vlin= %g (m/s); w=%g (deg/s); w_pl=%g (deg/s) \t (no-f) \n",
                SDI_F->servoingRef.numRef, 
                SDI_F->servoingRef.mu_occlu,SDI_F->servoingRef.mu_obst,
                SDI_F->servoingRef.occluding.docc,(double)METRICtoPIX(SDI_F->servoingRef.occluding.docc),
                SDI_F->servoingRef.occluding.dbord,(double)METRICtoPIX(SDI_F->servoingRef.occluding.dbord),
                SDI_F->servoingRef.obstacle.d_obst,SDI_F->servoingRef.ds_obst,
                RAD2DEG(SDI_F->servoingRef.obstacle.alpha),SDI_F->servoingRef.obstacle.avoidDirection,
								dot_qc.v, RAD2DEG(dot_qc.w), RAD2DEG(dot_qc.w_pl));
  
	if (filter){
		 // filtrage de la commande en acceleration... 
		if (fabs(dot_qc.v-servoingRef->dot_q.v)>SDI_F->robotParams.linAccMax)
			dot_qc.v = servoingRef->dot_q.v+sign(dot_qc.v-servoingRef->dot_q.v)*SDI_F->robotParams.linAccMax;
		else if (fabs(dot_qc.v)<SDI_F->robotParams.vMin) dot_qc.v =0.0;
		else	dot_qc.v =dot_qc.v;
		
		if (fabs(dot_qc.w-servoingRef->dot_q.w)>SDI_F->robotParams.angAccMax)
			dot_qc.w = servoingRef->dot_q.w+sign(dot_qc.w-servoingRef->dot_q.w)*SDI_F->robotParams.angAccMax;
		else if (fabs(dot_qc.w)<SDI_F->robotParams.wMin)	dot_qc.w =0.0;
		else dot_qc.w = dot_qc.w;
		
		if (fabs(dot_qc.w_pl-servoingRef->dot_q.w_pl)>SDI_F->robotParams.angAccMax)
			dot_qc.w_pl = servoingRef->dot_q.w_pl+sign(dot_qc.w_pl-servoingRef->dot_q.w_pl)*SDI_F->robotParams.angAccMax;
		else if (fabs(dot_qc.w_pl)<SDI_F->robotParams.wplMin)	dot_qc.w_pl =0.0;
    else	dot_qc.w_pl = dot_qc.w_pl;
	}


   //test d'arret
		((fabs(SDI_F->servoingRef.dot_q.v)<SDI_F->robotParams.vMin) && 
		(fabs(SDI_F->servoingRef.dot_q.w_pl)<SDI_F->robotParams.wplMin)&&
		(fabs(SDI_F->servoingRef.dot_q.w_pl)<SDI_F->robotParams.wplMin))
	 
	 //@todo stop if too slow?  

	servoingRef->dot_q = dot_qc;

  visuServPrintV(VISUSERV_VERBOSITY_VELOCITY,
								 "visuServMerge [#%d]: dot_q  ; vlin= %g ; w=%g ; w_pl=%g \t (f) \n",servoingRef->numRef, 
								 servoingRef->dot_q.v, RAD2DEG(servoingRef->dot_q.w), RAD2DEG(servoingRef->dot_q.w_pl));
	
  return OK;
}*/


/* ------------------------- Exec ---------------------------*/
STATUS visuservVisualServoingExecOLD(int *report)
{
  
  //ICU_VISUAL_DATA 	visualData=SDI_F->visualData,visualDataEst;
  VISUSERV_ROBOT_CONTROLS_STR 	dot_q, robotRef;    
  STATUS status;
  int offset=0, i,reply, need_estim=0, have_s=1;

  /* Robot state test (on goal or not) 
   if (SDI_F->state.mustStop==TRUE){
	SDI_F->state.visualServoingState=VISUSERV_SERVOING_MUST_STOP;		
	printf("visuservVisualServoingExec : robot receive stop request... \n");
	visualServoing_watchdog= 0;
	return END;  }

  // Robot base Orientation 
   if (visuservReadRobotConfig(robotSpeedPosterID, effectorSpeedPosterID, &(SDI_F->servoingRef.q), report) !=OK){
	printf("visuservVisualServoingExec : can't read robot configuration \n");
	return END; }*/

  /* Read visual data
    if (visuservReadVisualFeatures(&visualData,&reply) !=OK){
	have_s=0;
	if(reply==S_visuserv_USE_ESTIM){ //need_estim=(SDI_F->visuservParams.useEstimator>=VS_ODE_SCHEME_MIN);
	}
   	else if (reply != S_visuserv_NO_NEW_TARGET && reply !=S_visuserv_BAD_TARGET){
	*report = reply;
	printf("visuservVisualServoingExec: can't read visual data :(\n");
	return END; }
    else if (visualServoing_watchdog> VISUSERV_VISUALSERVOING_WATCHDOG){
	//need_estim=(SDI_F->visuservParams.useEstimator>=VS_ODE_SCHEME_MIN);
	// error : watchdog enabled!!!
	visualServoing_watchdog= 0;
	SDI_F->state.mustStop=TRUE;
	// le watchdog est activé...
	SDI_F->state.visualServoingState=VISUSERV_SERVOING_WATCHDOG;			
	*report = reply;
	printf("visuservVisualServoingExec : visualData is not updated too early !?");
	return END; }		
    } */


	/* Estimator 
	#ifndef VISUSERV_FORCE_ESTIMATOR

	if ((SDI_F->servoingRef.numRef>2)&&(sk.nbSpots!=0)&&
			((SDI_F->visualServoingParams.useEstimator>=VS_ODE_SCHEME_MIN)||(need_estim))){

		printf("VisualServoing: estimate visual data\n");
		status=visuservEstimVisualFeatures(qk,sk,dot_qk,(elapsedTime), &visualDataEst,&reply);
		if (status==VS_WUPDATE){
		printf("visuservVisualServoingExec: still updating the estimator...\n");
		need_estim=0;//disable estimation
		}else if (status!=OK){
		if (need_estim){
		printf("visuservVisualServoingExec: need estimate... but it failed!  :(\n");
		return END;
		}
			need_estim=0;
		} else
			need_estim=1;
	}else
		need_estim=0;
	#else
	
	printf("\n*** servoingRef.numRef:%d \t h:%gms\n",SDI_F->servoingRef.numRef,elapsedTime);
	for (i = 0; i < sk.nbSpots; i++){
	printf("\n*** (prev) sk%d: x:%g, y:%g, z:%g\n",i,
 	sk.x[i]/VISUSERV_TPIX+  VISUSERV_U0,sk.y[i]/VISUSERV_TPIX+ VISUSERV_V0,sk.z[i]);}
	printf("\n*** (prev) dot_qk: v:%g, w:%g, p:%g\n", dot_qk.v,dot_qk.w,dot_qk.w_pl);
		
	if ((SDI_F->servoingRef.numRef>2)){
	printf("\n*** (?prev?) dot_q: v:%g, w:%g, p:%g\n", SDI_F->servoingRef.dot_q.v,SDI_F->servoingRef.dot_q.w, 		SDI_F->servoingRef.dot_q.w_pl);
	status=visuservEstimVisualFeatures(qk,sk,dot_qk,(elapsedTime), &visualDataEst,&reply);
	if ((SDI_F->servoingRef.numRef>8)) return ETHER;
	}
	#endif
	

	// Robot base Orientation 
	if (visuservReadRobotConfig(robotSpeedPosterID, effectorSpeedPosterID, &(SDI_F->servoingRef.q),report)!=OK){
		printf("visuservVisualServoingExec : can't read robot configuration \n");
		return END;  }  

	// Robot velocity
        if(visuservReadRobotSpeed(&(SDI_F->robotSpeeds),report)!=OK)
		{printf("visuservVisualServoingExec : can't read robot speed\n ");
    		return END; } 

	if (have_s){
		sk=visualData;	
	}else if (need_estim){
		sk=visualDataEst;
	}
	if ((have_s)||(need_estim)){
		sk.numRef=++estimator_step;
		qk=SDI_F->servoingRef.q;
		//dot_qk=SDI_F->servoingRef.dot_q_rob;
		dot_qk=SDI_F->servoingRef.dot_q;
		//if (dot_qk.w_pl==0)  dot_qk.w_pl=SDI_F->servoingRef.dot_q.w_pl;

	for (i = 0; i < visualDataEst.nbSpots ; i++){
	printf("REAL? s_%d (%lg, %lg, %lg)\n", i,visualData.x[i]/VISUSERV_TPIX+  VISUSERV_U0, 
	visualData.y[i]/VISUSERV_TPIX+ VISUSERV_V0,visualData.z[i]);
	printf("ESTIM s_%d (%lg, %lg, %lg)\n", i,visualDataEst.x[i]/VISUSERV_TPIX+ VISUSERV_U0, 
	visualDataEst.y[i]/VISUSERV_TPIX+ VISUSERV_V0,visualDataEst.z[i]);
	printf("ERROR s_%d (%lg, %lg, %lg)\n", i,(visualData.x[i]-visualDataEst.x[i])/VISUSERV_TPIX, 
	(visualData.y[i]-visualDataEst.y[i])/VISUSERV_TPIX,(visualData.z[i]-visualDataEst.z[i]));
		}
	}

	if(((reply==S_visuserv_NO_NEW_TARGET)||(reply==S_visuserv_BAD_TARGET))&&(SDI_F->servoingRef.numRef>1)){	
		//else try to work in BO mode
		printf("\t\tWatchdog-%d \n",visualServoing_watchdog);
		visualServoing_watchdog++;      
		
		//decrease VS velocity
		SDI_F->servoingRef.dot_q_vs.v*=VISUSERV_RATIO_V;
		SDI_F->servoingRef.dot_q_vs.w*=VISUSERV_RATIO_W;
		//SDI_F->servoingRef.dot_q_vs.p*=VISUSERV_RATIO_P;
		SDI_F->state.visualServoingState=VISUSERV_SERVOING_WATCHDOG;
		have_s=0;}

	else	{printf(" VISUSERV_VERBOSITY_NOTICE_1,\t\tNO Watchdog-%d \n",visualServoing_watchdog);

		//No watchdog
		visualServoing_watchdog=0;
		
		//update the current visual data
		if (need_estim){
		printf("visuservVisualServoingExec: use the estimated visual data ---------------------------\n");
		SDI_F->visualData = visualDataEst;}
		else
		SDI_F->visualData = visualData;		

		#ifdef VISUSERV_VARIABLE_INTERACTION_MATRIX
		//SetCombinaisonMatrix(NULL );              
		GetCombinaisonMatrix(SDI_F->visualData ,NULL);  
		#endif
    	
	// VS contoller computation:
	if (visuservController(SDI_F->servoingRef.q.pp,NULL,SDI_F->visualDataRef,SDI_F->visualData, 
	SDI_F->visuservParams, &dot_q)!=OK){
		SDI_F->state.visualServoingState=VISUSERV_SERVOING_FAIL;      
		*report= S_visuserv_SERVOING_ERROR;      
		return END; }
		SDI_F->state.visualServoingState =VISUSERV_SERVOING_MOTION;
	}
	
	SDI_F->robotSpeeds=dot_q;

	/*update robot control
	if (visuservUpdateSpeed(SDI_F->servoingRef.dot_q, SDI_F->servoingRef.numRef, report)!=OK) return END;*/

      // Write control in posters
	SDI_F->robotRef.v = 0;    	
	SDI_F->robotRef.w = 0;
	SDI_F->effectorRef.pan = 0;    	
	SDI_F->effectorRef.tilt = 0;

  return ETHER;
}
