/*-------------------   visuservConst.h ------------------------------- 
 *  Auteur: David folio
 *  Mise a jour: Noureddine OUADAH
 *  Date : 03 02 2008
/*--------------------------------------------------------------------*/

#ifndef H_VISUSERVCONST
#define H_VISUSERVCONST

/*------------------------- Visuserv Posters Name ------------------------------*/

/*! Poster in which we save visuserv robot veloctiy  */
#define VISUSERV_DEFAULT_ROBOT_POSTERNAME	     	"Robot"

/*! Poster in which we save visuserv Effector velocity  */
#define VISUSERV_DEFAULT_EFFECTOR_POSTERNAME	     	"Effector"

/*------------------------- Imported Posters Name ------------------------------*/

/*! Poster containing proximitric sensor data*/
#define VISUSERV_DEFAULT_ECHO_POSTERNAME             	"rflexSonarCart"

/*! Poster containing Visual data */
#define VISUSERV_DEFAULT_VISUALDATA_POSTERNAME_ICU   	"icuVisualData"
#define VISUSERV_DEFAULT_VISUALDATA_POSTERNAME_HUMTRACK   	"HumTrackVisualData"

/*! Poster containing robot position and veloctiy  */
#define VISUSERV_DEFAULT_ROBOTSPEED_POSTERNAME	    	"rflexRobot"

/*! Poster containing Effector state  */
#define VISUSERV_DEFAULT_EFFECTORSPEED_POSTERNAME	"platineState" 
/*------------------------ Default State  ------------------------------*/

/*! Default visual servoing request state*/
#define VISUSERV_DEFAULT_VISUAL_SERVOING_STATE      VISUSERV_SERVOING_NO_INIT

/*! Default avoidance request state*/
#define VISUSERV_DEFAULT_AVOIDANCE_STATE            VISUSERV_NO_AVOIDANCE

#define VISUSERV_DEFAULT_USE_ESTIMATOR        -1

#define VISUSERV_EFFECTOR_STATE	           PLATINE_STATE 

//#define VISUSERV_ROBOT_CONTROL_STR	GENPOS_CART_SPEED
/*---------------------- Compatibility -----------------------------------*/
#define VISUSERV_TIMEZONE    NULL
/* from rflex module .....................................................*/
/*! Structure name for local module poster which contains speed to apply on robot */ 
//#define VISUSERV_SPEED_REF                 RFLEX_SPEED_REF_STR
/*! update period*/
#define VISUSERV_SPEED_UPDATE_PERIOD       5

/*! Which camera to use*/
#define VISUSERV_DEFAULT_IMAGEREF          0

/*--------------------- Robot settings -----------------------------------*/
 
/*! distance between Robot and effector centers
#define VISUSERV_Dx                   0.002		HUM2ICU */
#define VISUSERV_Dx                   0.2

/*! Camera height*/
#define VISUSERV_H_CAM                0.85

/*!X-coordinate of the camera center (in effector frame)*/
#define VISUSERV_Cx                   0.00

/*!Y-coordinate of the camera center (in effector frame)*/
#define VISUSERV_Cy                   0.00

/*! MAX vmax value */
#define VISUSERV_MAX_VMAX             1   //VMAX_ROBOT   m/s 

/*! MAX wmax value */
#define VISUSERV_MAX_WMAX             1   // WMAX_ROBOT   m/s 

/*! MAX effector velocity max value*/
#define VISUSERV_MAX_PMAX           (0.645772)  /* rd/s*/
#define VISUSERV_MAX_TMAX           (0.645772)  /* rd/s*/

/*! MAX effector position max value*/
#define VISUSERV_MAX_PPMAX           (82)  /* deg*/
#define VISUSERV_MAX_TPMAX           (31)  /* deg*/

/*! Defaul linear velocity max value in order to servoing saturation */
#define VISUSERV_DEFAULT_VMAX         0.25  /*  m/s */
/*! Default rotary robot velocity max value*/
#define VISUSERV_DEFAULT_WMAX         (M_PI_2)  /* rd/s*/
/*! Default effector velocity max value*/
#define VISUSERV_DEFAULT_PMAX       (VISUSERV_MAX_PMAX)  /* rd/s*/
#define VISUSERV_DEFAULT_TMAX       (VISUSERV_MAX_TMAX)  /* rd/s*/

/*! Defaul linear velocity min value in order to detect no move... */
#define VISUSERV_DEFAULT_VMIN         0.01  /*  m/s */
/*! Defaul angular velocity min value*/
#define VISUSERV_DEFAULT_WMIN         (0.1)  /*in rd/s*/
/*! Min effector velocity max value*/
#define VISUSERV_DEFAULT_PMIN       (0.01)  /* rd/s*/
#define VISUSERV_DEFAULT_TMIN       (0.01)  /* rd/s */

/*! MAX effector position min value*/
#define VISUSERV_MAX_PPMIN          (-85)  /* deg*/
#define VISUSERV_MAX_TPMIN           (-46)  /* deg*/

/*! Default nominal linear velocity */
#define VISUSERV_DEFAULT_V_NOMINAL    0.25  /*m/s*/
/*! Default nominal angular velocity */
#define VISUSERV_DEFAULT_W_NOMINAL    (M_PI/6)  /*rd/s*/
/*! Default nominal platine angular velocities */
#define VISUSERV_DEFAULT_P_NOMINAL  (M_PI/6)  /*rd/s*/
#define VISUSERV_DEFAULT_T_NOMINAL  (M_PI/6)  /*rd/s @@@@@@@@@@@@@@@@@@@@@@@@*/


/*! Defaul linear acceleration max value*/
#define VISUSERV_DEFAULT_LIN_ACC_MAX  0.2
/*! Acceleration max lineaire du robot */
#define VISUSERV_MAX_LIN_ACCMAX       1//SCOUT_AMAX_LIN_ROBOT
/*! Defaul rotary acceleration max value*/
#define VISUSERV_DEFAULT_ANG_ACC_MAX  0.785
/*! Acceleration max angulaire du robot */
#define VISUSERV_MAX_ANG_ACCMAX       1//SCOUT_AMAX_ANG_ROBOT
/*! Defaul linear deceleration max value*/
#define VISUSERV_DEFAULT_LIN_DEC_MAX  0.2
/*! Decceleration max lineaire du robot */
#define VISUSERV_MAX_LIN_DECMAX       1//SCOUT_AMAX_LIN_ROBOT
/*! Default rotary deceleration max value*/
#define VISUSERV_DEFAULT_ANG_DEC_MAX  1
/*! Acceleration max angulaire du robot */
#define VISUSERV_MAX_ANG_DECMAX       1//SCOUT_AMAX_ANG_ROBOT

/*! RATIO for robot linear velocity when watchdog is enabled*/
#define VISUSERV_RATIO_V              0.95
/*! RATIO for robot angular velocity when watchdog is enabled*/
#define VISUSERV_RATIO_W              0.95
/*! RATIO for platine angular velocity when watchdog is enabled*/
#define VISUSERV_RATIO_P            0.95
#define VISUSERV_RATIO_T            0.95

/*-------------------------- Visual Servoing settings -----------------------*/

/*!VisualData 'NO_NEW_TARGET'  watchdog limit.
 * This must be defined occording to the extraction time against the servoing time*/
#define VISUSERV_VISUALSERVOING_WATCHDOG    20
/*!VisualData end watchdog limit*/
#define VISUSERV_VISUALSERVOINGEND_WATCHDOG 4

/*! Default minimal linear velocity for visual servoing */
#define VISUSERV_DEFAULT_VISUAL_SERVOING_VMIN    0.025  /*  m/s */
/*! Default minimum error toreach the goal*/
#define VISUSERV_DEFAULT_VISUAL_SERVOING_EPSI    0.0005

/*! define the dimension max for lambda gain*/
#define VISUSERV_LAMBDA_DIM_MAX        3
/*! default \f$\lambda\f$-gain */
#define VISUSERV_DEFAULT_LAMBDA        1

/*! default \f$\lambda\f$-gain for visual servoing
 * (must be a user defined value with \ref SetVisualServoingParams)
 */
#define VISUSERV_DEFAULT_VISUAL_SERVOING_LAMBDA     VISUSERV_DEFAULT_LAMBDA
/*!default visual servoing \f$\lambda\f$-gain dimension */
#define VISUSERV_DEFAULT_VISUAL_SERVOING_LAMBDA_DIM 3

/*! Number maximum of visual data by target*/
#define VISUSERV_VISUALDATA_MAX       4//VISUSERV_DEFAULT_NB_SPOTS 

/*----------------------------------------------------------------------------*/
/*
 * Calibration data on scout
 * 
 * %-- Focal length (px):
 * fcpx = [ 768.523349558077371 ; 768.070283469532683 ];
 *  pix  =  7.4 e-6  % (m)
 * fcmm=fcpx*pix
 * %--> focal length (mm)
 * fcmm=[ 5.687072786729773e-03 ; 5.683720097674542e-03];
 * ratio : f_x / f_y = 1.000589875820346
 * %-- Principal point :
 * cc = [ 312.018060157157151 ; 253.286975726090645 ];
 *
 * final value :    
 * ==> f   = 5.685396442202157e-03 (m)  (mean value f_x et f_y)
 *		 u_0 = 2.532869757260906e+02 (px)
 *     v_0 = 3.120180601571572e+02 (px)
 */


/*! define the maximum value for docc distance*/
#define VISUSERV_DOCC_MAX_DISTANCE   (VISUSERV_VMAX*VISUSERV_TPIX)  

/*! define the maximum value for distance*/
#define VISUSERV_MAX_DISTANCE          HUGE_VAL

/*----------------------- Obstacles settings ---------------------------*/
/*! default obstacle radius...*/
#define VISUSERV_DEFAULT_OBST_RADIUS  0.175  /*m*/
/*! distance max to concider */
#define VISUSERV_OBST_DIST_MAX        1.5 /*m*/
/*! distance min before obstacle too near event */
#define VISUSERV_OBST_DIST_OBSTACLE_TOO_NEAR        0.17 /*m*/
/*! distance min to concider */
#define VISUSERV_OBST_DIST_MIN        VISUSERV_OBST_DIST_OBSTACLE_TOO_NEAR/*m*/

/*! default DF value  (application point)*/
#define VISUSERV_DEFAULT_DF           VISUSERV_Dx

/*! gap for secutity shape */
#define VISUSERV_OBST_D_e                  0.2  /* wrap gap*/
/*! default obstacle avoidance  \f$d_0\f$ wrap, which caracterize \f$\xi_{0}\f$
 * enveloppe*/ 
#define VISUSERV_DEFAULT_OBST_D_0          0.5  /* m */
/*! default obstacle avoidance  \f$d_-\f$ wrap, which caracterize \f$\xi_{-}\f$
 * enveloppe*/ 
#define VISUSERV_DEFAULT_OBST_D_minus      (VISUSERV_DEFAULT_OBST_D_0-VISUSERV_OBST_D_e)
/*! default obstacle avoidance  \f$d_+\f$ wrap, which caracterize \f$\xi_{+}\f$
 * enveloppe*/ 
#define VISUSERV_DEFAULT_OBST_D_plus       (VISUSERV_DEFAULT_OBST_D_0+VISUSERV_OBST_D_e)

/*! default obstacle avoidance gain */
#define VISUSERV_DEFAULT_OBST_K           0.2

/*! default obstacle avoidance redundancy task gain */
#define VISUSERV_DEFAULT_OBST_BETA        0.5 

/*! default obstacle avoidance reference linear velocity */
#define VISUSERV_DEFAULT_OBST_VREF        0.15 
/* default \f$\lambda\f$-gain for obstacle avoidance (must be a user defined value with \ref SetObstacleParams) */
#define VISUSERV_DEFAULT_OBST_LAMBDA     VISUSERV_DEFAULT_LAMBDA
/*!default obstacle \f$\lambda\f$-gain dimension */
#define VISUSERV_DEFAULT_OBST_LAMBDA_DIM 3

/*!default obstacle avoidance is in used value */
#define VISUSERV_DEFAULT_OBST_USE TRUE

/*------------------------- Occlusion settings ----------------------------*/
/*! default occlusion avoidance  \f$D_0\f$ wrap, which caracterize \f$\Xi_{0}\f$  enveloppe*/ 
#define VISUSERV_DEFAULT_OCC_D_0        1  
/*(70.0*VISUSERV_TPIX/ VISUSERV_FOCUS)*/
/*! default occlusion avoidance  \f$D_-\f$ wrap, which caracterize \f$\Xi_{-}\f$  enveloppe*/ 
#define VISUSERV_DEFAULT_OCC_D_minus    1  
/*(40.0*VISUSERV_TPIX/ VISUSERV_FOCUS)*/
/*! default occlusion avoidance  \f$D_+\f$ wrap, which caracterize \f$\Xi_{+}\f$  enveloppe*/ 
#define VISUSERV_DEFAULT_OCC_D_plus     1
/*(80.0*VISUSERV_TPIX/ VISUSERV_FOCUS)*/

/*! default occlusion avoidance redundancy task gain */
#define VISUSERV_DEFAULT_OCC_BETA        8

/*! default \f$\lambda\f$-gain for occlusion avoidance
 * (must be a user defined value with \ref SetOcclusionParams)
 */
#define VISUSERV_DEFAULT_OCC_LAMBDA     VISUSERV_DEFAULT_LAMBDA
/*!default occlusion \f$\lambda\f$-gain dimension */
#define VISUSERV_DEFAULT_OCC_LAMBDA_DIM 3


/*!default obstacle avoidance is in used value */
#define VISUSERV_DEFAULT_OCC_USE        TRUE

/*! default @a Avoid  type*/
#define VISUSERV_DEFAULT_AVOID_TYPE     VISUSERV_AVOID_OCC_OBS

/*------------------------- Miscellaneous definition  --------------------*/
#ifndef TRUE
/*! Define True value */
#define TRUE     1
#endif

#ifndef FALSE
/*! define false value*/
#define FALSE    0
#endif

/*! Max string size*/
#define VISUSERV_MAX_STRLGTH        256

#ifndef M_PI_2
/*! if math library is not used */
#define M_PI_2                  1.5708
#endif 
#ifndef M_PI
/*! Definition locale de la valeur de \f$ \pi\f$.
 * On ne la definit que si \a M_PI de math.h n'est pas definit.
 */
#define  M_PI                   3.1415927
#endif

#ifndef min
/*! return the min value between \a a and \a b*/
#define min(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef max
/*! return the max value between \a a and \a b*/
#define max(a,b) ((a)>(b)?(a):(b))
#endif

/*! Allow to easily compute the @a x square (eg: x^2*/
#define VS_SQUARE(x)  ((x)*(x))

/*! Allow to easily compute the euclidian distance betwen the point A(x0,y0) and
 * B(x,y)*/ 
#define VS_DISTANCE(x0,y0,x,y)  (sqrt((double)VS_SQUARE(x-x0)+(double)VS_SQUARE(y-y0)))

#ifndef RAD2DEG
/*! Convert RAD -> DEG */
#define RAD2DEG(rad)            (rad*180/M_PI)
#endif
#ifndef DEG2RAD
/*! Convert DEG -> RAD */
#define DEG2RAD(deg)            (deg*M_PI/180)
#endif

/** This macro allow to easily convert data in 'pixel' to metric */
#define METRICtoPIX(x)                  (((double) (x))/(VISUSERV_TPIX))
/*#define METRICtoPIX(x)                (x*VISUSERV_FOCUS/VISUSERV_TPIX)*/
/** This macro allow to easily convert data in  metric to 'pixel'  */
#define PIXtoMETRIC(x)                  (((double) (x))*(VISUSERV_TPIX))
/*#define PIXtoMETRIC(x)                (x*VISUSERV_TPIX/VISUSERV_FOCUS)*/


/** This macro allow to easily convert data in  metric to 'pixel' for U coordinate  */
#define METRICtoPIX_U(x)                (((double) (x))*(VISUSERV_KU))
/** This macro allow to easily convert data in  metric to 'pixel' for V coordinate  */
#define METRICtoPIX_V(x)                (-((double) (x))*(VISUSERV_KV))
/** This macro allow to easily convert data in  metric to 'pixel'  */
#define PIX_UtoMETRIC(x)                (((double) (x))/(VISUSERV_KU))
/** This macro allow to easily convert data in  metric to 'pixel'  */
#define PIX_VtoMETRIC(x)                (-((double) (x))/(VISUSERV_KV))

#endif /* H_VISUSERVCONST */

/* on peut couper la poire en deux  */
