/*  ------------------------- visuservStruct.h ---------------------------
 *  Auteur: David folio
 *  Mise a jour: Noureddine OUADAH
 *  Date : 03 02 2008
 *------------------------------------------------------------------------*/

#ifndef H_VISUSERVSTRUCT
#define H_VISUSERVSTRUCT

/*! Needs constant definition*/
#include "visuservConst.h"

/*! Posters names and IDs  lists */

typedef struct VISUSERV_POSTERS_STR {
char robotPosterName[VISUSERV_MAX_STRLGTH];   		/* robot position and veloctiy poster name*/
char effectorPosterName[VISUSERV_MAX_STRLGTH];   	/* effector position and veloctiy poster name*/
char robotSpeedPosterName[VISUSERV_MAX_STRLGTH];   	/* robot position and veloctiy poster name*/
char effectorSpeedPosterName[VISUSERV_MAX_STRLGTH];   	/* effector position and veloctiy poster name*/	
char echoPosterName[VISUSERV_MAX_STRLGTH];       	/* echo poster name*/
char visualDataPosterName[VISUSERV_MAX_STRLGTH]; 	/* visual data poster name*/
} VISUSERV_POSTERS_STR;

typedef struct VISUSERV_POSTERSID_STR {
int robotPosterID;   		/* robot position and veloctiy poster name*/
int effectorPosterID;   	/* effector position and veloctiy poster name*/
int robotSpeedPosterID;   	/* robot position and veloctiy poster name*/
int effectorSpeedPosterID;   	/* effector position and veloctiy poster name*/	
int echoPosterID;      		/* echo poster name*/
int visualDataPosterID; 	/* visual data poster name*/
} VISUSERV_POSTERSID_STR;

/* ---------------------------Robot structure --------------------------*/
/* Robot configuration : This data structure provide robots position, orientation and effector state*/
typedef struct VISUSERV_ROBOT_CONFIGURATION_STR {
	double x;          	/* robot x coordinate*/
	double y;         	/* robot y coordinate */
	double theta;     	/* robot orientation  */
	double vr;         	/* velovity dotx coordinate*/  
	double wr;         	/* velovity doty coordinate*/ 
	double l;		/* curvilinear robot position  */
	double pp;         	/* platine horizontal position  */
	double tp;         	/* platine vertical position */
}VISUSERV_ROBOT_CONFIGURATION_STR;

/* Robot control : This data structure provide robots velocity control (linear, angular velocity)*/
typedef struct VISUSERV_ROBOT_CONTROLS_STR {
	double vr;         // robot linear velocity  
	double wr;         // robot angular velocity 
	double p;         // platine horizontal angular velocity 
	double t;         // platine vertical angular velocity 
}VISUSERV_ROBOT_CONTROLS_STR;

/* Robot parameters :This structure contains the main robot kynematics, dynamics parameters */
typedef struct VISUSERV_ROBOT_PARAMS_STR {
	double vMax;         /* maximal linear velocity */
	double wMax;         /* maximal angular velocity  */
	double vMin;         /* minimal linear velocity */
	double wMin;         /* minimal angular velocity  */
	double pMax;         /* maximal pan position for effector  */
	double tMax;         /* maximal tilt position for effector  */
	double pMin;         /* minimal pan position for effector */
	double tMin;         /* minimal tilt position for effector  */
	double pVMax;        /* maximal pan velocity for effector  */
	double tVMax;        /* maximal tilt velocity for effector  */
	double pVMin;        /* minimal pan velocity for effector */
	double tVMin;        /* minimal tilt velocity for effector  */
	double linAccMax;    /* maximal linear acceleration */
	double angAccMax;    /* maximal angular acceleration */
	double linDecMax;    /* maximal linear deceleration  */
	double angDecMax;    /* maximal angular decelerat<ion */
}VISUSERV_ROBOT_PARAMS_STR;

/* --------------------------- Effecor structure (Platine)--------------------------*/

/*  Effecor configuration */
typedef struct VISUSERV_PLATINE_CONFIGURATION_STR {
	double pan;  	/* horizontal effector orientation  */
	double tilt;  	/* vertical effector orientation */
}VISUSERV_PLATINE_CONFIGURATION_STR;

/*  Effecor control variables */
typedef struct VISUSERV_PLATINE_CONTROL_STR {
	double pan;      /* effec horizontal velocity */
	double tilt;     /* effec vertical velovity*/
}VISUSERV_PLATINE_CONTROL_STR;

/*  Effecor parameters */
typedef struct VISUSERV_PLATINE_PARAMS_STR {
	double panVMax;        /* maximal pan angular velocity   */
	double tiltVMax;       /* maximal tilt angular velocity  */
	double panVMin;        /* minimal pan angular velocity    */
	double tiltVMin;       /* minimal tilt angular velocity  */
}VISUSERV_PLATINE_PARAMS_STR;

typedef struct VISUSERV_ROBOT_GAINS_STR {
	double Lv;     /* Gain for linear robot speed control*/
	double Lw;     /* Gain for angular robot speed control  */
	double Lp;     /* Gain for pan speed effector control */
}VISUSERV_ROBOT_GAINS_STR;

/* ---------------------------------- obstacle structure ---------------------------------*/

/* Obstacle features  */
typedef struct VISUSERV_OBSTACLE_DATA_STR{
	double d_obst;     /* distance robot/obstacle (m)*/
	double alpha;      /* orientaion robot/obstacle (rd)*/
	double radius;     /* obstacle curvatuve radius*/
	int avoidDirection;/* obstacle avoid direction. Must be +1 or -1. */
	int numRef;        /* data reference number */
} VISUSERV_OBSTACLE_DATA_STR;

/*!\ Obstacle parameters : This structure define the obstacles parameters (such as security wrap...) and
 * obstacles avoidance controller parameters (such as gain...)*/
typedef struct VISUSERV_OBSTACLE_PARAMS_STR {
	double DF;              /* repulsive force application point */
	double D_0;             /* \f$d_0\f$ wrap, which caracterize \f$\xi_{0}\f$	enveloppe */
	double D_minus;         /* \f$d_-\f$ wrap, which caracterize \f$\xi_{-}\f$	enveloppe */
	double D_plus;          /* \f$d_+\f$ wrap, which caracterize \f$\xi_{+}\f$	enveloppe */
	double beta;            /* obstacle avoidance redundancy task gain */
	double vref;            /* reference linear velocity*/
	double k;               /* obstacle avoidance gain, must be a positive value*/
/*! \f$\lambda\f$-gain for obstacle avoidance.
 * The \f$\lambda\f$-gain is a positive  scalar or a positive definite
 * matrix. Then you must set \a lamda_dim properly.
 * If you plan to use matrix (\a lambda_dim >1), we assume to use a diagonal
 * matrix, for example if \a lambda_dim=3:
 *\f[\Lambda = \left[\begin{array}{ccc}\lambda_1 & 0 & 0 \\0 &\lambda_2 & 0  \\ 0 & 0 & \lambda_3 \\ \end{array}\right]\f]*/
	double lambda[VISUSERV_LAMBDA_DIM_MAX];    
	int lambda_dim;         /* define lambda dim (usually 1 or 3)*/
	int useObstacleAvoid;   /* This allow to enable (or disable) obstacle avoidance controller.
	* \attention This value is overwrited by the \a Avoid request argument \sa VISUSERV_AVOID_ARGS_STR*/
}VISUSERV_OBSTACLE_PARAMS_STR;

/* Rotational Potential request parameters : This structure the obstacles parameters, 
 * such as security wrap..unused yet, plan to used Rotational Potential here?*/
typedef struct VISUSERV_ROTATIONAL_POTENTIAL_STR {
	double Kv;              /* static gain constant for linear velocity*/
	double Kw;              /* static gain constant for angular velocity*/
	double Kwpl;            /* static gain constant for the platine angular velocity*/
}VISUSERV_ROTATIONAL_POTENTIAL_STR;

/*------------------- Visual features: occluding object -----------------------------------------*/
/* Visual data : the target Structure which contains the visual occluding object (\f$\mathcal O\f$  )
 * information.  
 * \attention : In this structur the distance (docc and dbord) is assume to be the
 * \b signed distance! todo for further extension may be need xocc and zocc? (but not yet needed) */
typedef struct VISUSERV_VISUAL_OCCLUDING_STR {
	double xc;                       /* X center coordinate*/
	double yc;                       /* Y center coordinate*/
	double zc;                       /* Z center coordinate*/
	double width;                    /* object width*/
	double height;                   /* object height*/
	double ys;                       /* caracterize the closer target side */
	double zs;                       /* caracterize the closer target depth */
	double yocc;                     /* occluding side */
	double zocc;                     /* occluding side depth */
	double docc;                     /* \b Signed occluding distance */
	double dbord;                    /* \b Signed edge distance */
	int numRef;                      /* Reference counter*/
	int unused;                      /* not used */
}VISUSERV_VISUAL_OCCLUDING_STR;

/* Occlusion parameters : This structure define the occlusion parameters (such as security wrap...) and
 * occlusion avoidance controller parameters (such as gain...)*/
typedef struct VISUSERV_OCCLUDING_PARAMS_STR{
	double D_0;             /* \f$D_0\f$ wrap, which caracterize \f$\Xi_{0}\f$ enveloppe */
	double D_minus;         /* \f$D_{-}\f$ wrap, which caracterize \f$\Xi_{-}\f$ enveloppe*/
	double D_plus;          /* \f$D_{+}\f$ wrap, which caracterize \f$\Xi_{+}\f$ enveloppe */
	double beta;            /* occlusion avoidance redundancy task gain */
/*! \f$\lambda\f$-gain for obstacle avoidance.
 * The \f$\lambda\f$-gain is a positive  scalar or a positive definite
 * matrix. Then you must set \a lamda_dim properly.
 * If you plan to use matrix (\a lambda_dim >1), we assume to use a diagonal
 * matrix, for example if \a lambda_dim=3:
*\f[\Lambda = \left[\begin{array}{ccc}\lambda_1 & 0 & 0 \\0 &\lambda_2 & 0  \\ 0 & 0 & \lambda_3 \\ \end{array}\right]\f]*/
	double lambda[VISUSERV_LAMBDA_DIM_MAX];    
	int lambda_dim;         /* define lambda dim (usually 1 or 3)*/
	int useOclusionAvoid;   /* This allow to turn enable (or disable) occlusion avoidance controller.
	* \attention This value is overwrited by the \a Avoid request argument \sa VISUSERV_AVOID_ARGS_STR */	
}VISUSERV_OCCLUDING_PARAMS_STR;

/* ---------------------Avoid request -------------------------------------------------------------*/
/*! Provide avoidance strategy type*/
typedef enum _VISUSERV_AVOID_TYPE {
	VISUSERV_AVOID_NONE=-1,       /* No avoidance strategy*/
	VISUSERV_AVOID_VSRP=1,        /* rotationnal potential avoidance strategy.
	 * \attention There are no occlusion avoidance! \unused Unused yet, plan to used Rotational Potential here?*/
	VISUSERV_AVOID_OCCLUSION,     /* occlusion avoidance only*/
	VISUSERV_AVOID_OCCLUSION_OBS, /* occlusion avoidance with obstacle avoidance cost function */
	VISUSERV_AVOID_OCC_OBS        /* Both occlusion and obstacle avoidance*/
} VISUSERV_AVOID_TYPE;

/*! Avoid request argument data structure*/
typedef struct VISUSERV_AVOID_ARGS_STR{
	VISUSERV_AVOID_TYPE type;	  // avoidance strategy type but it's just used to move robot with specific speeds 
	/*add more ...*/
} VISUSERV_AVOID_ARGS_STR;


/* ---------------Visual Servoing -----------------------------------------------------------*/
typedef enum VISUSERV_TYPES {
  VISUSERV_BASIC,			/* classical VS : e_dot = - lambda.e */
  VISUSERV_DYNAMIC,			/* Dynamic VS : considers ds/dt */  
  VISUSERV_SLIDING_CONTROLER,		/* Sliding mode VS : s_dot.s < 0  */
  VISUSERV_COMBINED    			/* combination between Sliding mode and Classical */
} VISUSERV_TYPES;

typedef enum VISUSERV_MODES {
  VISUSERV_CONSTANT_INTER_MTRX,		/* L = L (s of desired position ) */
  VISUSERV_VARIABLE_INTER_MTRX,		/* L = L (s mesured each iteration) */
  VISUSERV_BOTH 			/* L = 1/2 ( Ls mesured + Ls desired) */
} VISUSERV_MODES;

typedef enum VISUSERV_SLIDING_TYPES {
  VISUSERV_SLIDING_1_SIGN,				/* q_dot = K Sign(s) */
  VISUSERV_SLIDING_2_SAT,				/* q_dot = K Sat(s/phi) */
  VISUSERV_SLIDING_3_SMOOTH				/* q_dot = K Smooth(s,phi) */
}  VISUSERV_SLIDING_TYPES;

typedef enum VISUSERV_TRACKMODES {
  VISUSERV_TRACK_PANTILT,		/* human track by PTU only */
  VISUSERV_TRACK_WHOLEROBOT		/* human track by robot movment*/
} VISUSERV_TRACKMODES; 

/* Visual Servoing parameters : This structure contains the visual servoing  parameters.*/
typedef struct VISUSERV_VISUAL_SERVOING_PARAMS_STR {
	double epsi;             			/* Minimum error to reach the goal*/ 
	double lambda[3]; 				/* visuserv gains matrix*/   
	VISUSERV_MODES 			vs_mode;	/* have a look above */
        VISUSERV_TYPES 			vs_type;		/* "  */
        VISUSERV_SLIDING_TYPES 		sm_type;		/* "  */
}VISUSERV_VISUAL_SERVOING_PARAMS_STR;

typedef struct VISUSERV_TRACK_PARAMS_STR {
 	VISUSERV_TRACKMODES  tr_mode;	/* human track types */
	double panKp;       		/* Proportional Gain for pan speed control*/
	double panKi;       		/* Integral Gain for pan speed control   */
	double panKd;       		/* Derivative gain for pan speed control */
	double tiltKp;      		/* Proportional Gain for tilt speed control*/
	double tiltKi;      		/* Integral Gain for tilt speed control  */
	double tiltKd;      		/* Derivative gain for tilt speed control */
	double vKp;       		/* Proportional Gain for linear speed control*/
	double vKi;       		/* Integral Gain for linear speed control   */
	double vKd;       		/* Derivative gain for linear speed control */
	double wKp;      		/* Proportional Gain for angular speed control*/
	double wKi;      		/* Integral Gain for angular speed control  */
	double wKd;      		/* Derivative gain for angular speed control */
}VISUSERV_TRACK_PARAMS_STR;

/* ---------------State/internal info -------------------------------------------------------*/
/* Servoing info : This structure provide internal information about the current robot
 * configuration (q),  the different controllers values (dot_q_xx) and risks value */
typedef struct VISUSERV_SERVOING_REF_STR {
	VISUSERV_ROBOT_CONFIGURATION_STR q;      /* the current robot configuration*/
  	VISUSERV_ROBOT_CONTROLS_STR dot_q;        /* the global controller*/
  	VISUSERV_ROBOT_CONTROLS_STR dot_qc;       /* applied controler @note adding for debug purpose may be removed*/
	VISUSERV_ROBOT_CONTROLS_STR dot_q_vs;     /* the visual servoing controller*/
	VISUSERV_ROBOT_CONTROLS_STR dot_q_obst;   /* the obstacle avoidance controller*/
	VISUSERV_ROBOT_CONTROLS_STR dot_q_occlu;  /* the occlusion avoidance controller*/
	VISUSERV_OBSTACLE_DATA_STR    obstacle;  /* Obstacle features*/
	VISUSERV_VISUAL_OCCLUDING_STR occluding; /* Obstacle features*/
	double mu_obst;  			 /* obstacle collision risk: a obstacle distance function*/ 
	double mu_occlu; 			 /* occlusion risk: a occlusion distance function*/ 
	/*! Occlusion avoidance flag. This flag is set to @a TRUE when the occlusion
	 * risk has reached its maximum value (ie \f$ \mu_{\mathrm{occlu}}=1\f$) until
	 * the @a LEAVE_OCCLU condition has been fullfilled. */	
	int AVOID_OCCLU;
	/*!Leave occlusion flag. This flag is to @a TRUE when the leave occlusion
	 * condition has been fullfilled */	
	int LEAVE_OCCLU;
	/*!Occlusion disctance  when the leave occlusion condition has been fullfilled. */
	double ds_occlu;
	/*! Collision avoidance flag. This flag is set to @a TRUE when the collision
	 * risk has reached its maximum value (ie \f$ \mu_{\mathrm{obst}}=1\f$) until
	 * the @a LEAVE_OCCLU condition has been fullfilled. */	
	int AVOID_OBST;
	/*!Leave collision flag. This flag is to @a TRUE when the leave obstacle
	 * condition has been fullfilled	 */	
	int LEAVE_OBST;
	/*!Collision disctance  when the leave obstacle condition has been fullfilled.*/
	double ds_obst;
  	double samplingTime;    /* An estimatation of sampling time (in ms) If set to 0 reset sampling time*/
  	int numRef;             /* counting step*/
	int unused;             /* unused*/
} VISUSERV_SERVOING_REF_STR;

/*! Define robot servoing/motion state */
typedef enum VISUSERV_SERVOING_STATE_ENUM{
	/* Error state (eg fatal error)*/
	VISUSERV_SERVOING_FAIL,          /* An error occur and motion activity stopped*/
	/* Warn state*/
	VISUSERV_SERVOING_ISMOVING,      /* robot's still moving. \note this state is different against \a VISUSERV_SERVOING_MOTION, which is an ordered motion. Here this state is a "warning": robot is moving by an other request*/
	VISUSERV_SERVOING_MUST_STOP,   /* Robot has a request for stop,so the robot must or had to stop*/
	VISUSERV_SERVOING_WATCHDOG,    /* watchdog enabled!!*/
	VISUSERV_SERVOING_NO_INIT,     /* (default) need servoing INIT request can't send request before init*/ 
	/* "safe" state'*/
	VISUSERV_SERVOING_ISSTOPING,     /* Robot is stopping*/
	VISUSERV_SERVOING_STOPPED,       /* Robot is off*/
	VISUSERV_SERVOING_MOTION         /* Robot  move */
}VISUSERV_SERVOING_STATE;

/*! Define avoidance state */
typedef enum VISUSERV_AVOIDANCE_STATE_ENUM{
	VISUSERV_AVOID_FAIL,       	/* An error occur and avoid activity stopped*/
	VISUSERV_OBSTACLE_TOO_NEAR,    	/* a too close obstacle is detected*/
	VISUSERV_OCCLUDING_TOO_NEAR,   	/* a too close occluding object is detected*/
	VISUSERV_OCCLUDED_PARTIALLY,   	/* the target is partially occluded*/
	VISUSERV_OCCLUDED_TOTALY,      	/* the target is (totaly!) occluded*/
	VISUSERV_NO_AVOIDANCE,         	/* the @a Avoid request is not started*/
	VISUSERV_AVOID_ISRUNNING       	/* The avoidance strategy is running*/
}VISUSERV_AVOIDANCE_STATE;

/*  Module State : Structure which contains module state variable */
typedef struct VISUSERV_STATE_STR {
	int initDone;     	         		/* True if module has been initialized   */
	int targetRefIsDefined;      			/* True if a reference target is defined */
	int onGoal;                 	 		/* True if robot is on goal */
	double TF[3];					/* Task function (e)*/
	double TFP[3];					/* variations of Task function (e_dot)*/
	double KSM[3];					/* computed control gains (for sliding mode)*/
	int mustStop;             			/* True if robot must stop*/  
	VISUSERV_SERVOING_STATE visualServoingState;  	/* Visual Servoing request state*/
	VISUSERV_AVOIDANCE_STATE avoidanceState;      	/* Avoid request state*/
} VISUSERV_STATE_STR;

#endif /* H_VISUSERVSTRUCT */
